package org.openxma.dsl.reference.base.dto;

import java.io.Serializable;
import org.openxma.dsl.platform.dao.EntityFactory;
import org.openxma.dsl.platform.service.MapperContextHolder;
import java.util.Date;
import org.openxma.dsl.platform.service.EntityMapperSupport;
import org.openxma.dsl.reference.base.model.BaseEntity;

public abstract class BaseEntityViewGen implements Serializable,EntityMapperSupport {
	private static final long serialVersionUID = 351580434;
	protected String oid;
	protected Date version;
	
	public String getOid() {
        return this.oid;
    }
	
	public void setOid(String oid) {
        this.oid=oid;
    }
	
	public Date getVersion() {
        return this.version;
    }
	
	public void setVersion(Date version) {
        this.version=version;
    }
	
    public void mapFrom(Object sourceObject) {
    	if (sourceObject instanceof BaseEntity) {
    		BaseEntity baseEntity = (BaseEntity) sourceObject;
    		internalMapFromBaseEntity(baseEntity);	
    	} 
    	else if (sourceObject != null) {
			throw new IllegalArgumentException("mapping from object '"+sourceObject+"' is not supported");
		}
	}
	
	public <T> T mapTo(Class<T> target) {
		if (BaseEntityView.class.equals(target)) { 
			BaseEntityView baseEntityView = new BaseEntityView();
			internalMapToBaseEntityView(baseEntityView);		
			return target.cast(baseEntityView);
		}
		else if (BaseEntity.class.equals(target)) {
    		return target.cast(internalMapToBaseEntity(target));
    	}
    	else { 
    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
    	}
    }
	
    public void mapTo(Object targetObject) {
    	if (targetObject instanceof BaseEntityView) { 
			internalMapToBaseEntityView(((BaseEntityView)targetObject));		
		}
    	else if (targetObject instanceof BaseEntity) {
    		internalMapToBaseEntity(((BaseEntity)targetObject));	
    	} 
    	else if (targetObject != null) { 
			throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
		}
	}
    
    protected void internalMapToBaseEntityView(BaseEntityView baseEntityView) {
		baseEntityView.setOid(getOid());
		baseEntityView.setVersion(getVersion());
    }
    
    protected BaseEntity internalMapToBaseEntity(Class<?> target) {
    	EntityFactory<BaseEntity> entityFactory = (EntityFactory<BaseEntity>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(BaseEntity.class);
    	BaseEntity baseEntity = (BaseEntity) 
    		entityFactory.createEntity(null);
    	internalMapToBaseEntity(baseEntity);	
    	return baseEntity;
    }
    
    protected void internalMapToBaseEntity(BaseEntity baseEntity) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, baseEntity);
			if (getOid() != null) {
				baseEntity.setOid(getOid());
			}
			baseEntity.setVersion(getVersion());
	    }
	}
	
	protected void internalMapFromBaseEntity(BaseEntity baseEntity) {
		if (!MapperContextHolder.getContext().containsKey(baseEntity)) {
	    	MapperContextHolder.getContext().put(baseEntity, this);			
			setOid(baseEntity.getOid());
			setVersion(baseEntity.getVersion());
		}
	}
    
    @Override
    public String toString() {
    	return new StringBuilder("BaseEntityView [")
        	.append("oid=").append(getOid()).append(",")
        	.append("version=").append(getVersion())
        	.append("]")
            .toString();
    }
}
