package org.openxma.dsl.reference.dao;

import org.openxma.dsl.platform.dao.*;
import java.util.Collection;
import java.util.Date;
import org.openxma.dsl.reference.base.ContactInfo;
import org.openxma.dsl.reference.dao.CustomerDao;
import org.openxma.dsl.reference.dto.CustomerContactReport;
import org.openxma.dsl.reference.dto.CustomerIdDto;
import org.openxma.dsl.reference.dto.CustomerView;
import org.openxma.dsl.reference.model.Address;
import org.openxma.dsl.reference.model.Customer;
import org.openxma.dsl.reference.types.Gender;

/**
 * Represents a Data Access Object, as defined in the
 * Core J2EE Pattern Catalog (see http://java.sun.com/blueprints/corej2eepatterns/Patterns/DataAccessObject.html),
 * to abstract and encapsulate all access to entity CustomerDao.
 * @see CustomerDao
 */
public interface CustomerDaoGen extends GenericDao<Customer,String>, EntityFactory<Customer> {
    public static final SortOrderSpecification LAST_NAME_ASC = SortOrderSpecification.create("Customer.LastNameAsc").asc("lastName");
    /**
     * Factory method to create an empty <code>Customer</code> entity with an already assigned identifier.
     * The advantage of this approach, coupled with the fact that we're using the id for equals,
     * is that equality test's are immediately possible and therefore independent from the persistence state
     * of the entity. This in contrast to other solutions where auto generated Keys are used
     * which must wait until the object is persisted or differentiate between transient and managed states.
     * @return a new (transient), empty <code>Customer</code> entity with an already assigned identifier.
     */
    Customer create();
    /**
     * Factory method to create an empty <code>Customer</code> taking all not null attributes.
     *
	 * @param invoiceAddress
	 * @param deliveryAddress
	 * @param customerFirstName
	 * @param customerLastName
	 * @param gender
	 * @param emailAddress
	 * @param customerBirthDate
     * @return a new (transient ) <code>Customer</code> entity with the given natural identifier and not null attributes set.
     */
    Customer create(Address invoiceAddress,Address deliveryAddress,String customerFirstName,String customerLastName,Gender gender,String emailAddress,Date customerBirthDate);
    /**
     * Factory method to create and save an empty <code>Customer</code> taking all not null attributes.
     *
	 * @param invoiceAddress
	 * @param deliveryAddress
	 * @param customerFirstName
	 * @param customerLastName
	 * @param gender
	 * @param emailAddress
	 * @param customerBirthDate
     * @return a new (transient) <code>Customer</code> entity with the given natural identifier and not null attributes set.
     */
    Customer createAndSave(Address invoiceAddress,Address deliveryAddress,String customerFirstName,String customerLastName,Gender gender,String emailAddress,Date customerBirthDate);
    /**
     * Return the persistent instance of the Customer entity class
     * with the given natural identifier or <code>null</code> if not found.
     *
	 * @param customerFirstName
	 * @param customerLastName
	 * @param customerBirthDate
     * @return the persistent Customer instance, or <code>null</code> if not found
     */
     Customer loadByCustomerNk(String customerFirstName,String customerLastName,Date customerBirthDate);
    /**
     * Return the persistent instance of the Customer entity class
     * with the given unique-key or <code>null</code> if not found.
     *
	 * @param emailAddress
     * @return the persistent Customer instance, or <code>null</code> if not found
     */
     Customer loadByCustomerUkEmail(String emailAddress);
    /**
     *
     * @param firstName
     * @param lastName
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findAllByFirstNameAndLastName(String firstName,String lastName);
    /**
     *
     * @param lastName
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findByNamedQueryCustomerLastName(String lastName);
    /**
     *
     * @param lastName
     * @param firstResult
     * @param maxResults
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findAllLikeLastNamePageable(String lastName,Integer firstResult,Integer maxResults);
    /**
     *
     * @param lastName
     * @param firstResult
     * @param maxResults
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findAllInName(Collection<String> lastName,Integer firstResult,Integer maxResults);
    /**
     *
     * @param birthDate
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findByBirthDate(Date birthDate);
    /**
     *
     * @return a collection containing 0 or more persistent CustomerContactReport instances
     */
    Collection<CustomerContactReport> loadCustomerContactReport();
    /**
     *
     * @return a collection containing 0 or more persistent CustomerIdDto instances
     */
    Collection<CustomerIdDto> findAllCustomerIds();
    /**
     *
     * @param name
     * @return a collection containing 0 or more persistent String instances
     */
    Collection<String> findAllCustomerIdLikeName(String name);
    /**
     *
     * @param contactInfo
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findCustomerByContact(ContactInfo contactInfo);
    /**
     *
     * @param customerView
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findByCustomerView(CustomerView customerView);
    /**
     *
     * @param customer
     * @return a collection containing 0 or more persistent Customer instances
     */
    Collection<Customer> findCustomerByExample(Customer customer);
}
