package org.openxma.dsl.reference.dao;

import org.openxma.dsl.platform.dao.*;
import java.util.Collection;
import java.util.Date;
import org.openxma.dsl.reference.dao.OrderDao;
import org.openxma.dsl.reference.dto.OrderStateView;
import org.openxma.dsl.reference.model.Customer;
import org.openxma.dsl.reference.model.Order;
import org.openxma.dsl.reference.types.valueobject.OrderNumber;

/**
 * Represents a Data Access Object, as defined in the
 * Core J2EE Pattern Catalog (see http://java.sun.com/blueprints/corej2eepatterns/Patterns/DataAccessObject.html),
 * to abstract and encapsulate all access to entity OrderDao.
 * @see OrderDao
 */
public interface OrderDaoGen extends GenericDao<Order,String>, EntityFactory<Order> {
    public static final SortOrderSpecification DATE_ASC = SortOrderSpecification.create("Order.DateAsc").asc("placementDate");
    public static final SortOrderSpecification AMOUNT_ASC = SortOrderSpecification.create("Order.AmountAsc").asc("totalAmount");
    /**
     * Factory method to create an empty <code>Order</code> entity with an already assigned identifier.
     * The advantage of this approach, coupled with the fact that we're using the id for equals,
     * is that equality test's are immediately possible and therefore independent from the persistence state
     * of the entity. This in contrast to other solutions where auto generated Keys are used
     * which must wait until the object is persisted or differentiate between transient and managed states.
     * @return a new (transient), empty <code>Order</code> entity with an already assigned identifier.
     */
    Order create();
    /**
     * Factory method to create an empty <code>Order</code> taking all not null attributes.
     *
	 * @param customer
	 * @param orderNumber
	 * @param orderStoreId
	 * @param orderPlacementDate
	 * @param orderState
	 * @param totalAmount
     * @return a new (transient ) <code>Order</code> entity with the given natural identifier and not null attributes set.
     */
    Order create(Customer customer,OrderNumber orderNumber,String orderStoreId,Date orderPlacementDate,String orderState,Double totalAmount);
    /**
     * Factory method to create and save an empty <code>Order</code> taking all not null attributes.
     *
	 * @param customer
	 * @param orderNumber
	 * @param orderStoreId
	 * @param orderPlacementDate
	 * @param orderState
	 * @param totalAmount
     * @return a new (transient) <code>Order</code> entity with the given natural identifier and not null attributes set.
     */
    Order createAndSave(Customer customer,OrderNumber orderNumber,String orderStoreId,Date orderPlacementDate,String orderState,Double totalAmount);
    /**
     * Return the persistent instance of the Order entity class
     * with the given natural identifier or <code>null</code> if not found.
     *
	 * @param customerFirstName
	 * @param customerLastName
	 * @param customerBirthDate
	 * @param orderStoreId
	 * @param orderPlacementDate
     * @return the persistent Order instance, or <code>null</code> if not found
     */
     Order loadByOrderNk(String customerFirstName,String customerLastName,Date customerBirthDate,String orderStoreId,Date orderPlacementDate);
    /**
     * Return the persistent instance of the Order entity class
     * with the given unique-key or <code>null</code> if not found.
     *
	 * @param customer
	 * @param orderNumber
     * @return the persistent Order instance, or <code>null</code> if not found
     */
     Order loadByOrderUkOrderNumber(Customer customer,OrderNumber orderNumber);
    /**
     * Return the persistent instance of the Order entity class
     * with the given unique-key or <code>null</code> if not found.
     *
	 * @param customerOid
	 * @param orderNumber
     * @return the persistent Order instance, or <code>null</code> if not found
     */
     Order loadByOrderUkOrderNumber(String customerOid,OrderNumber orderNumber);
    /**
     *
     * @param customerId
     * @param reportDate
     * @return a collection containing 0 or more persistent OrderStateView instances
     */
    Collection<OrderStateView> readOrderStateByCustomerAndDate(Long customerId,Date reportDate);
    /**
     *
     * @param order
     * @param from
     * @param to
     * @return a collection containing 0 or more persistent Order instances
     */
    Collection<Order> findAllOrderStatesBetweenDate(Order order,Date from,Date to);
    /**
     *
     * @param customer
     * @return a collection containing 0 or more persistent Order instances
     */
    Collection<Order> findAllByCustomer(Customer customer);
}
