package org.openxma.dsl.reference.dao;

import org.openxma.dsl.platform.dao.*;
import java.util.Date;
import java.util.Set;
import org.openxma.dsl.reference.base.CreditCardInfo;
import org.openxma.dsl.reference.dao.PrivateCustomerDao;
import org.openxma.dsl.reference.model.Address;
import org.openxma.dsl.reference.model.PrivateCustomer;
import org.openxma.dsl.reference.types.Gender;

/**
 * Represents a Data Access Object, as defined in the
 * Core J2EE Pattern Catalog (see http://java.sun.com/blueprints/corej2eepatterns/Patterns/DataAccessObject.html),
 * to abstract and encapsulate all access to entity PrivateCustomerDao.
 * @see PrivateCustomerDao
 */
public interface PrivateCustomerDaoGen extends GenericDao<PrivateCustomer,String>, EntityFactory<PrivateCustomer> {
    /**
     * Factory method to create an empty <code>PrivateCustomer</code> entity with an already assigned identifier.
     * The advantage of this approach, coupled with the fact that we're using the id for equals,
     * is that equality test's are immediately possible and therefore independent from the persistence state
     * of the entity. This in contrast to other solutions where auto generated Keys are used
     * which must wait until the object is persisted or differentiate between transient and managed states.
     * @return a new (transient), empty <code>PrivateCustomer</code> entity with an already assigned identifier.
     */
    PrivateCustomer create();
    /**
     * Factory method to create an empty <code>PrivateCustomer</code> taking all not null attributes.
     *
	 * @param invoiceAddress
	 * @param deliveryAddress
	 * @param customerFirstName
	 * @param customerLastName
	 * @param gender
	 * @param emailAddress
	 * @param customerBirthDate
	 * @param ccInfo
     * @return a new (transient ) <code>PrivateCustomer</code> entity with the given natural identifier and not null attributes set.
     */
    PrivateCustomer create(Address invoiceAddress,Address deliveryAddress,String customerFirstName,String customerLastName,Gender gender,String emailAddress,Date customerBirthDate,CreditCardInfo ccInfo);
    /**
     * Factory method to create and save an empty <code>PrivateCustomer</code> taking all not null attributes.
     *
	 * @param invoiceAddress
	 * @param deliveryAddress
	 * @param customerFirstName
	 * @param customerLastName
	 * @param gender
	 * @param emailAddress
	 * @param customerBirthDate
	 * @param ccInfo
     * @return a new (transient) <code>PrivateCustomer</code> entity with the given natural identifier and not null attributes set.
     */
    PrivateCustomer createAndSave(Address invoiceAddress,Address deliveryAddress,String customerFirstName,String customerLastName,Gender gender,String emailAddress,Date customerBirthDate,CreditCardInfo ccInfo);
    /**
     * Return the persistent instance of the PrivateCustomer entity class
     * with the given natural identifier or <code>null</code> if not found.
     *
	 * @param customerFirstName
	 * @param customerLastName
	 * @param customerBirthDate
     * @return the persistent PrivateCustomer instance, or <code>null</code> if not found
     */
     PrivateCustomer loadByCustomerNk(String customerFirstName,String customerLastName,Date customerBirthDate);
    /**
     * Execute a query based on the given index attributes.
     *
	 * @param customerFirstName
	 * @param customerLastName
	 * @param customerBirthDate
     * @param firstResult the index of the first result object to be retrieved (numbered from 0 or null if ignored)
     * @param maxResults the maximum number of result objects to retrieved (null or <=0 for no limit)
     * @return a {@link Set} containing 0 or more persistent PrivateCustomer instances
     */
     Set<PrivateCustomer> findByNameFilter(String customerFirstName,String customerLastName,Date customerBirthDate,Integer firstResult,Integer maxResults);
}
