package org.openxma.dsl.reference.dao.impl;

import org.openxma.dsl.platform.dao.*;
import org.openxma.dsl.platform.dao.impl.*;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import org.openxma.dsl.platform.config.XProperty;
import org.openxma.dsl.reference.base.ContactInfo;
import org.openxma.dsl.reference.dao.CustomerDao;
import org.openxma.dsl.reference.dto.CustomerContactReport;
import org.openxma.dsl.reference.dto.CustomerIdDto;
import org.openxma.dsl.reference.dto.CustomerView;
import org.openxma.dsl.reference.model.Address;
import org.openxma.dsl.reference.model.Customer;
import org.openxma.dsl.reference.model.impl.CustomerImpl;
import org.openxma.dsl.reference.types.Gender;
/**
 * @see CustomerDao
 */
@SuppressWarnings("unused")
public abstract class CustomerDaoGenImpl extends GenericDaoHibernateImpl<Customer,String> implements CustomerDao {
	protected Integer jdbcBatchSize;
	
	@XProperty(key="jdbcBatchSize")
	public void setJdbcBatchSize(Integer jdbcBatchSize) {
        this.jdbcBatchSize=jdbcBatchSize;
    }
    /**
     * @param sessionFactory the sessionFactory to set
     */
    @Autowired
    public void setSessionFactory(@Qualifier("default") SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }
    /**
     * @param dataFieldMaxValueIncrementer the dataFieldMaxValueIncrementer to set
     */
    @Autowired
    public void setDataFieldMaxValueIncrementer(@Qualifier("default")
            DataFieldMaxValueIncrementer dataFieldMaxValueIncrementer) {
        this.dataFieldMaxValueIncrementer = dataFieldMaxValueIncrementer;
    }

    public CustomerDaoGenImpl() {
        super(CustomerImpl.class);
    }
    
    /**
     * @param context the context for the object creation
     * @return a transient instance of entity <code>Customer</code> with an already assigned identifier.
     */
    public Customer createEntity(Object context) {
        return create();
    }

    public Customer create()
    {
        CustomerImpl customer = new CustomerImpl();
        customer.setOid(dataFieldMaxValueIncrementer.nextStringValue());
        return customer;
    }

    public Customer create(Address invoiceAddress,Address deliveryAddress,String customerFirstName,String customerLastName,Gender gender,String emailAddress,Date customerBirthDate) {
        Assert.notNull(invoiceAddress, "Parameter 'invoiceAddress' must not be null");
        Assert.notNull(deliveryAddress, "Parameter 'deliveryAddress' must not be null");
        Assert.notNull(customerFirstName, "Parameter 'customerFirstName' must not be null");
        Assert.notNull(customerLastName, "Parameter 'customerLastName' must not be null");
        Assert.notNull(gender, "Parameter 'gender' must not be null");
        Assert.notNull(emailAddress, "Parameter 'emailAddress' must not be null");
        Assert.notNull(customerBirthDate, "Parameter 'customerBirthDate' must not be null");
        Customer customer = (Customer) create();
        customer.setInvoiceAddress(invoiceAddress);
        customer.setDeliveryAddress(deliveryAddress);
        customer.setFirstName(customerFirstName);
        customer.setLastName(customerLastName);
        customer.setGender(gender);
        customer.setEmailAddress(emailAddress);
        customer.setBirthDate(customerBirthDate);
        return customer;
    }
    
    public Customer createAndSave(Address invoiceAddress,Address deliveryAddress,String customerFirstName,String customerLastName,Gender gender,String emailAddress,Date customerBirthDate) {
        Customer customer = create(invoiceAddress,deliveryAddress,customerFirstName,customerLastName,gender,emailAddress,customerBirthDate);
        saveOrUpdate(customer);
        return customer;
    }
    
    public Customer loadByCustomerNk(String customerFirstName,String customerLastName,Date customerBirthDate) {
        Criterion criterion = equal("firstName", customerFirstName, false);
        criterion = and(criterion,"lastName", customerLastName, false);
        criterion = and(criterion,"birthDate", customerBirthDate, false);
        Criteria criteria = createCriteria(criterion);
        return (Customer) unique(criteria);
    }

    public Customer loadByCustomerUkEmail(String emailAddress) {
        Criterion criterion = equal("emailAddress", emailAddress, false);
        Criteria criteria = createCriteria(criterion);
        return (Customer) unique(criteria);
    }
    	
    public Collection<Customer> findAllByFirstNameAndLastName(String firstName,String lastName) {
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery("Customer.FindAllByFirstNameAndLastName");
        namedQuery.setParameter("firstName", firstName);
        namedQuery.setParameter("lastName", lastName);
        applyFindAllByFirstNameAndLastNameQueryHints(namedQuery,firstName,lastName);
        return list(namedQuery);
    }
    
    protected void applyFindAllByFirstNameAndLastNameQueryHints(Query query,String firstName,String lastName) {
   		// override this method and apply other hints that influence how a query is executed
    }
    	
    public Collection<Customer> findByNamedQueryCustomerLastName(String lastName) {
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery("Customer.FindByNamedQueryCustomerLastName");
        namedQuery.setParameter("lastName", lastName);
        applyFindByNamedQueryCustomerLastNameQueryHints(namedQuery,lastName);
        return list(namedQuery);
    }
    
    protected void applyFindByNamedQueryCustomerLastNameQueryHints(Query query,String lastName) {
   		// override this method and apply other hints that influence how a query is executed
    }
    	
    public Collection<Customer> findAllLikeLastNamePageable(String lastName,Integer firstResult,Integer maxResults) {
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery("Customer.FindAllLikeLastNamePageable");
        namedQuery.setParameter("lastName", lastName);
        applyFindAllLikeLastNamePageableQueryHints(namedQuery,lastName,firstResult,maxResults);
        return list(namedQuery);
    }
    
    protected void applyFindAllLikeLastNamePageableQueryHints(Query query,String lastName,Integer firstResult,Integer maxResults) {
        if (firstResult!=null) {
        	query.setFirstResult(firstResult);
        }
        if (maxResults!=null) {
        	query.setMaxResults(maxResults);
        }
   		// override this method and apply other hints that influence how a query is executed
    }
    	
    public Collection<Customer> findAllInName(Collection<String> lastName,Integer firstResult,Integer maxResults) {
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery("Customer.FindAllInName");
        namedQuery.setParameterList("lastName", lastName);
        applyFindAllInNameQueryHints(namedQuery,lastName,firstResult,maxResults);
        return list(namedQuery);
    }
    
    protected void applyFindAllInNameQueryHints(Query query,Collection<String> lastName,Integer firstResult,Integer maxResults) {
        if (firstResult!=null) {
        	query.setFirstResult(firstResult);
        }
        if (maxResults!=null) {
        	query.setMaxResults(maxResults);
        }
   		// override this method and apply other hints that influence how a query is executed
    }
    	
    public Collection<Customer> findByBirthDate(Date birthDate) {
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery("Customer.FindByBirthDate");
        namedQuery.setDate("birthDate", birthDate);
        applyFindByBirthDateQueryHints(namedQuery,birthDate);
        return list(namedQuery);
    }
    
    protected void applyFindByBirthDateQueryHints(Query query,Date birthDate) {
   		// override this method and apply other hints that influence how a query is executed
    }
    	
    public Collection<CustomerContactReport> loadCustomerContactReport() {
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery("Customer.LoadCustomerContactReport");
        applyLoadCustomerContactReportQueryHints(namedQuery);
        int rowNumber=0;
        Collection<CustomerContactReport> result = new ArrayList<CustomerContactReport>();
        for (Object object : list(namedQuery)) {
        	Object[] tuple = (Object[]) (object instanceof Object[] ? object: new Object[]{object});
        	CustomerContactReport customerContactReport = mapLoadCustomerContactReportTuple(tuple, rowNumber++);
        	if (customerContactReport!=null) {
        		result.add(customerContactReport);
        	}
		}
		return result;
    }
    
    protected void applyLoadCustomerContactReportQueryHints(Query query) {
   		// override this method and apply other hints that influence how a query is executed
    }
    
    protected CustomerContactReport mapLoadCustomerContactReportTuple(Object[] tuple, int rowNumber) {
    	CustomerContactReport customerContactReport = new CustomerContactReport();
    	customerContactReport.setFirstName((String)tuple[0]);
    	customerContactReport.setLastName((String)tuple[1]);
    	customerContactReport.setEmailAddress((String)tuple[2]);
    	return customerContactReport;
    }
    	
    public Collection<CustomerIdDto> findAllCustomerIds() {
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery("Customer.FindAllCustomerIds");
        applyFindAllCustomerIdsQueryHints(namedQuery);
        int rowNumber=0;
        Collection<CustomerIdDto> result = new ArrayList<CustomerIdDto>();
        for (Object object : list(namedQuery)) {
        	Object[] tuple = (Object[]) (object instanceof Object[] ? object: new Object[]{object});
        	CustomerIdDto customerIdDto = mapFindAllCustomerIdsTuple(tuple, rowNumber++);
        	if (customerIdDto!=null) {
        		result.add(customerIdDto);
        	}
		}
		return result;
    }
    
    protected void applyFindAllCustomerIdsQueryHints(Query query) {
   		// override this method and apply other hints that influence how a query is executed
    }
    
    protected CustomerIdDto mapFindAllCustomerIdsTuple(Object[] tuple, int rowNumber) {
    	CustomerIdDto customerIdDto = new CustomerIdDto();
    	customerIdDto.setOid((String)tuple[0]);
    	return customerIdDto;
    }
    	
    public Collection<String> findAllCustomerIdLikeName(String name) {
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery("Customer.FindAllCustomerIdLikeName");
        namedQuery.setParameter("name", name);
        applyFindAllCustomerIdLikeNameQueryHints(namedQuery,name);
        return list(namedQuery);
    }
    
    protected void applyFindAllCustomerIdLikeNameQueryHints(Query query,String name) {
   		// override this method and apply other hints that influence how a query is executed
    }
    	
    public Collection<Customer> findCustomerByContact(ContactInfo contactInfo) {
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery("Customer.FindCustomerByContact");
        namedQuery.setParameter("contactInfo_email", contactInfo.getEmail());
        applyFindCustomerByContactQueryHints(namedQuery,contactInfo);
        return list(namedQuery);
    }
    
    protected void applyFindCustomerByContactQueryHints(Query query,ContactInfo contactInfo) {
   		// override this method and apply other hints that influence how a query is executed
    }
    	
    public Collection<Customer> findByCustomerView(CustomerView customerView) {
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery("Customer.FindByCustomerView");
        if (customerView.getEmailAddress() != null) {
        	namedQuery.setParameter("customerView_emailAddress", customerView.getEmailAddress());
        }
        applyFindByCustomerViewQueryHints(namedQuery,customerView);
        return list(namedQuery);
    }
    
    protected void applyFindByCustomerViewQueryHints(Query query,CustomerView customerView) {
   		// override this method and apply other hints that influence how a query is executed
    }
    	
    public Collection<Customer> findCustomerByExample(Customer customer) {
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery("Customer.FindCustomerByExample");
        if (customer.getEmailAddress() != null) {
        	namedQuery.setParameter("customer_emailAddress", customer.getEmailAddress());
        }
        if (customer.getFirstName() != null) {
        	namedQuery.setParameter("customer_firstName", customer.getFirstName());
        }
        if (customer.getLastName() != null) {
        	namedQuery.setParameter("customer_lastName", customer.getLastName());
        }
        applyFindCustomerByExampleQueryHints(namedQuery,customer);
        return list(namedQuery);
    }
    
    protected void applyFindCustomerByExampleQueryHints(Query query,Customer customer) {
   		// override this method and apply other hints that influence how a query is executed
    }
}
