package org.openxma.dsl.reference.dao.impl;

import org.openxma.dsl.platform.dao.*;
import org.openxma.dsl.platform.dao.impl.*;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;
import java.util.Date;
import org.openxma.dsl.reference.dao.OrderItemDao;
import org.openxma.dsl.reference.model.Order;
import org.openxma.dsl.reference.model.OrderItem;
import org.openxma.dsl.reference.model.Product;
import org.openxma.dsl.reference.model.impl.OrderItemImpl;
/**
 * @see OrderItemDao
 */
@SuppressWarnings("unused")
public abstract class OrderItemDaoGenImpl extends GenericDaoHibernateImpl<OrderItem,String> implements OrderItemDao {
    /**
     * @param sessionFactory the sessionFactory to set
     */
    @Autowired
    public void setSessionFactory(@Qualifier("default") SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }
    /**
     * @param dataFieldMaxValueIncrementer the dataFieldMaxValueIncrementer to set
     */
    @Autowired
    public void setDataFieldMaxValueIncrementer(@Qualifier("default")
            DataFieldMaxValueIncrementer dataFieldMaxValueIncrementer) {
        this.dataFieldMaxValueIncrementer = dataFieldMaxValueIncrementer;
    }

    public OrderItemDaoGenImpl() {
        super(OrderItemImpl.class);
    }
    
    /**
     * @param context the context for the object creation
     * @return a transient instance of entity <code>OrderItem</code> with an already assigned identifier.
     */
    public OrderItem createEntity(Object context) {
        return create();
    }

    public OrderItem create()
    {
        OrderItemImpl orderItem = new OrderItemImpl();
        orderItem.setOid(dataFieldMaxValueIncrementer.nextStringValue());
        return orderItem;
    }

    public OrderItem create(Order order,Product product,Integer itemNo,Long quantity) {
        Assert.notNull(order, "Parameter 'order' must not be null");
        Assert.notNull(product, "Parameter 'product' must not be null");
        Assert.notNull(itemNo, "Parameter 'itemNo' must not be null");
        Assert.notNull(quantity, "Parameter 'quantity' must not be null");
        OrderItem orderItem = (OrderItem) create();
        orderItem.setOrder(order);
        orderItem.setProduct(product);
        orderItem.setItemNo(itemNo);
        orderItem.setQuantity(quantity);
        return orderItem;
    }
    
    public OrderItem createAndSave(Order order,Product product,Integer itemNo,Long quantity) {
        OrderItem orderItem = create(order,product,itemNo,quantity);
        saveOrUpdate(orderItem);
        return orderItem;
    }
    
    public OrderItem loadByOrderItemNk(String customerFirstName,String customerLastName,Date customerBirthDate,String orderStoreId,Date orderPlacementDate,Integer itemNo) {
        Criterion criterion = equal("customer.firstName", customerFirstName, false);
        criterion = and(criterion,"customer.lastName", customerLastName, false);
        criterion = and(criterion,"customer.birthDate", customerBirthDate, false);
        criterion = and(criterion,"order.storeId", orderStoreId, false);
        criterion = and(criterion,"order.placementDate", orderPlacementDate, false);
        criterion = and(criterion,"itemNo", itemNo, false);
        Criteria criteria = createCriteria(criterion);
        criteria.createAlias("customer","customer");
        criteria.createAlias("order","order");
        return (OrderItem) unique(criteria);
    }
}
