package org.openxma.dsl.reference.dao.impl;

import org.openxma.dsl.platform.dao.*;
import org.openxma.dsl.platform.dao.impl.*;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;
import java.util.Date;
import java.util.LinkedHashSet;
import java.util.Set;
import org.openxma.dsl.reference.base.CreditCardInfo;
import org.openxma.dsl.reference.dao.PrivateCustomerDao;
import org.openxma.dsl.reference.model.Address;
import org.openxma.dsl.reference.model.PrivateCustomer;
import org.openxma.dsl.reference.model.impl.PrivateCustomerImpl;
import org.openxma.dsl.reference.types.Gender;
/**
 * @see PrivateCustomerDao
 */
@SuppressWarnings("unused")
public abstract class PrivateCustomerDaoGenImpl extends GenericDaoHibernateImpl<PrivateCustomer,String> implements PrivateCustomerDao {
    /**
     * @param sessionFactory the sessionFactory to set
     */
    @Autowired
    public void setSessionFactory(@Qualifier("default") SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }
    /**
     * @param dataFieldMaxValueIncrementer the dataFieldMaxValueIncrementer to set
     */
    @Autowired
    public void setDataFieldMaxValueIncrementer(@Qualifier("default")
            DataFieldMaxValueIncrementer dataFieldMaxValueIncrementer) {
        this.dataFieldMaxValueIncrementer = dataFieldMaxValueIncrementer;
    }

    public PrivateCustomerDaoGenImpl() {
        super(PrivateCustomerImpl.class);
    }
    
    /**
     * @param context the context for the object creation
     * @return a transient instance of entity <code>PrivateCustomer</code> with an already assigned identifier.
     */
    public PrivateCustomer createEntity(Object context) {
        return create();
    }

    public PrivateCustomer create()
    {
        PrivateCustomerImpl privateCustomer = new PrivateCustomerImpl();
        privateCustomer.setOid(dataFieldMaxValueIncrementer.nextStringValue());
        return privateCustomer;
    }

    public PrivateCustomer create(Address invoiceAddress,Address deliveryAddress,String customerFirstName,String customerLastName,Gender gender,String emailAddress,Date customerBirthDate,CreditCardInfo ccInfo) {
        Assert.notNull(invoiceAddress, "Parameter 'invoiceAddress' must not be null");
        Assert.notNull(deliveryAddress, "Parameter 'deliveryAddress' must not be null");
        Assert.notNull(customerFirstName, "Parameter 'customerFirstName' must not be null");
        Assert.notNull(customerLastName, "Parameter 'customerLastName' must not be null");
        Assert.notNull(gender, "Parameter 'gender' must not be null");
        Assert.notNull(emailAddress, "Parameter 'emailAddress' must not be null");
        Assert.notNull(customerBirthDate, "Parameter 'customerBirthDate' must not be null");
        Assert.notNull(ccInfo, "Parameter 'ccInfo' must not be null");
        PrivateCustomer privateCustomer = (PrivateCustomer) create();
        privateCustomer.setInvoiceAddress(invoiceAddress);
        privateCustomer.setDeliveryAddress(deliveryAddress);
        privateCustomer.setFirstName(customerFirstName);
        privateCustomer.setLastName(customerLastName);
        privateCustomer.setGender(gender);
        privateCustomer.setEmailAddress(emailAddress);
        privateCustomer.setBirthDate(customerBirthDate);
        privateCustomer.setCcInfo(ccInfo);
        return privateCustomer;
    }
    
    public PrivateCustomer createAndSave(Address invoiceAddress,Address deliveryAddress,String customerFirstName,String customerLastName,Gender gender,String emailAddress,Date customerBirthDate,CreditCardInfo ccInfo) {
        PrivateCustomer privateCustomer = create(invoiceAddress,deliveryAddress,customerFirstName,customerLastName,gender,emailAddress,customerBirthDate,ccInfo);
        saveOrUpdate(privateCustomer);
        return privateCustomer;
    }
    
    public PrivateCustomer loadByCustomerNk(String customerFirstName,String customerLastName,Date customerBirthDate) {
        Criterion criterion = equal("firstName", customerFirstName, false);
        criterion = and(criterion,"lastName", customerLastName, false);
        criterion = and(criterion,"birthDate", customerBirthDate, false);
        Criteria criteria = createCriteria(criterion);
        return (PrivateCustomer) unique(criteria);
    }

    public Set<PrivateCustomer> findByNameFilter(String customerFirstName,String customerLastName,Date customerBirthDate,Integer firstResult,Integer maxResults) {
        Criterion criterion = equal("firstName", customerFirstName, false);
        criterion = and(criterion,"lastName", customerLastName, false);
        criterion = and(criterion,"birthDate", customerBirthDate, false);
        Criteria criteria = createCriteria(criterion);
        applyPagingParameters(criteria,firstResult,maxResults);
        return new LinkedHashSet<PrivateCustomer>(list(criteria));
    }
}
