package org.openxma.dsl.reference.dao.impl;

import org.openxma.dsl.platform.dao.*;
import org.openxma.dsl.platform.dao.impl.*;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;
import org.openxma.dsl.reference.dao.ProductDao;
import org.openxma.dsl.reference.model.Product;
import org.openxma.dsl.reference.model.Supplier;
import org.openxma.dsl.reference.model.impl.ProductImpl;
import org.openxma.dsl.reference.types.valueobject.ProductId;
/**
 * @see ProductDao
 */
@SuppressWarnings("unused")
public abstract class ProductDaoGenImpl extends GenericDaoHibernateImpl<Product,ProductId> implements ProductDao {
    /**
     * @param sessionFactory the sessionFactory to set
     */
    @Autowired
    public void setSessionFactory(@Qualifier("default") SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }
    /**
     * @param dataFieldMaxValueIncrementer the dataFieldMaxValueIncrementer to set
     */
    @Autowired
    public void setDataFieldMaxValueIncrementer(@Qualifier("default")
            DataFieldMaxValueIncrementer dataFieldMaxValueIncrementer) {
        this.dataFieldMaxValueIncrementer = dataFieldMaxValueIncrementer;
    }

    public ProductDaoGenImpl() {
        super(ProductImpl.class);
    }
    
    /**
     * @param context the context for the object creation
     * @return a transient instance of entity <code>Product</code> with an already assigned identifier.
     */
    public Product createEntity(Object context) {
        return create();
    }

    public Product create()
    {
        ProductImpl product = new ProductImpl();
        product.setOid(new ProductId(dataFieldMaxValueIncrementer.nextStringValue()));
        return product;
    }

    public Product create(Supplier supplier,String name,Integer unitPrice) {
        Assert.notNull(supplier, "Parameter 'supplier' must not be null");
        Assert.notNull(name, "Parameter 'name' must not be null");
        Assert.notNull(unitPrice, "Parameter 'unitPrice' must not be null");
        Product product = (Product) create();
        product.setSupplier(supplier);
        product.setName(name);
        product.setUnitPrice(unitPrice);
        return product;
    }
    
    public Product createAndSave(Supplier supplier,String name,Integer unitPrice) {
        Product product = create(supplier,name,unitPrice);
        saveOrUpdate(product);
        return product;
    }
}
