package org.openxma.dsl.reference.dao.impl;

import org.openxma.dsl.platform.dao.*;
import org.openxma.dsl.platform.dao.impl.*;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;
import java.util.Collection;
import org.openxma.dsl.reference.dao.SupplierDao;
import org.openxma.dsl.reference.dto.SupplierNameView;
import org.openxma.dsl.reference.model.Supplier;
import org.openxma.dsl.reference.model.impl.SupplierImpl;
/**
 * @see SupplierDao
 */
@SuppressWarnings("unused")
public abstract class SupplierDaoGenImpl extends GenericDaoHibernateImpl<Supplier,String> implements SupplierDao {
    /**
     * @param sessionFactory the sessionFactory to set
     */
    @Autowired
    public void setSessionFactory(@Qualifier("default") SessionFactory sessionFactory) {
        this.sessionFactory = sessionFactory;
    }
    /**
     * @param dataFieldMaxValueIncrementer the dataFieldMaxValueIncrementer to set
     */
    @Autowired
    public void setDataFieldMaxValueIncrementer(@Qualifier("default")
            DataFieldMaxValueIncrementer dataFieldMaxValueIncrementer) {
        this.dataFieldMaxValueIncrementer = dataFieldMaxValueIncrementer;
    }

    public SupplierDaoGenImpl() {
        super(SupplierImpl.class);
    }
    
    /**
     * @param context the context for the object creation
     * @return a transient instance of entity <code>Supplier</code> with an already assigned identifier.
     */
    public Supplier createEntity(Object context) {
        return create();
    }

    public Supplier create()
    {
        SupplierImpl supplier = new SupplierImpl();
        supplier.setOid(dataFieldMaxValueIncrementer.nextStringValue());
        return supplier;
    }

    public Supplier create(String companyName) {
        Assert.notNull(companyName, "Parameter 'companyName' must not be null");
        Supplier supplier = (Supplier) create();
        supplier.setCompanyName(companyName);
        return supplier;
    }
    
    public Supplier createAndSave(String companyName) {
        Supplier supplier = create(companyName);
        saveOrUpdate(supplier);
        return supplier;
    }
    
    public Supplier loadBySupplierKey(String companyName) {
        Criterion criterion = equal("companyName", companyName, false);
        Criteria criteria = createCriteria(criterion);
        return (Supplier) unique(criteria);
    }
    	
    public Collection<SupplierNameView> findAllByCompanyName(String companyName) {
        Assert.notNull(companyName, "Parameter 'companyName' must not be null");
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery("Supplier.FindAllByCompanyName");
        namedQuery.setParameter(0, companyName);
        applyFindAllByCompanyNameQueryHints(namedQuery,companyName);
        return list(namedQuery);
    }
    
    protected void applyFindAllByCompanyNameQueryHints(Query query,String companyName) {
   		//apply other hints that influence how a query is executed
    }
    	
    public Long getSupplierCount() {
        Query namedQuery = this.sessionFactory.getCurrentSession().getNamedQuery("Supplier.GetSupplierCount");
        applyGetSupplierCountQueryHints(namedQuery);
        return unique(namedQuery);
    }
    
    protected void applyGetSupplierCountQueryHints(Query query) {
   		//apply other hints that influence how a query is executed
    }
}
