package org.openxma.dsl.reference.dto;

import java.io.Serializable;
import org.openxma.dsl.platform.dao.EntityFactory;
import org.openxma.dsl.platform.service.MapperContextHolder;
import org.openxma.dsl.platform.service.EntityMapperSupport;
import org.openxma.dsl.reference.model.Bug;

public abstract class BugViewGen implements Serializable,EntityMapperSupport {
	private static final long serialVersionUID = 93984613;
	protected Long oid;
	protected String bugId;
	
	public Long getOid() {
        return this.oid;
    }
	
	public void setOid(Long oid) {
        this.oid=oid;
    }
	
	public String getBugId() {
        return this.bugId;
    }
	
	public void setBugId(String bugId) {
        this.bugId=bugId;
    }
	
    public void mapFrom(Object sourceObject) {
    	if (sourceObject instanceof Bug) {
    		Bug bug = (Bug) sourceObject;
    		internalMapFromBug(bug);	
    	} 
    	else if (sourceObject != null) {
			throw new IllegalArgumentException("mapping from object '"+sourceObject+"' is not supported");
		}
	}
	
	public <T> T mapTo(Class<T> target) {
		if (BugView.class.equals(target)) { 
			BugView bugView = new BugView();
			internalMapToBugView(bugView);		
			return target.cast(bugView);
		}
		else if (Bug.class.equals(target)) {
    		return target.cast(internalMapToBug(target));
    	}
    	else { 
    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
    	}
    }
	
    public void mapTo(Object targetObject) {
    	if (targetObject instanceof BugView) { 
			internalMapToBugView(((BugView)targetObject));		
		}
    	else if (targetObject instanceof Bug) {
    		internalMapToBug(((Bug)targetObject));	
    	} 
    	else if (targetObject != null) { 
			throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
		}
	}
    
    protected void internalMapToBugView(BugView bugView) {
		bugView.setOid(getOid());
		bugView.setBugId(getBugId());
    }
    
    protected Bug internalMapToBug(Class<?> target) {
    	EntityFactory<Bug> entityFactory = (EntityFactory<Bug>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(Bug.class);
    	Bug bug = (Bug) 
    		entityFactory.createEntity(null);
    	internalMapToBug(bug);	
    	return bug;
    }
    
    protected void internalMapToBug(Bug bug) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, bug);
			if (getOid() != null) {
				bug.setOid(getOid());
			}
			bug.setBugId(getBugId());
	    }
	}
	
	protected void internalMapFromBug(Bug bug) {
		if (!MapperContextHolder.getContext().containsKey(bug)) {
	    	MapperContextHolder.getContext().put(bug, this);			
			setOid(bug.getOid());
			setBugId(bug.getBugId());
		}
	}
    
    @Override
    public String toString() {
    	return new StringBuilder("BugView [")
        	.append("oid=").append(getOid()).append(",")
        	.append("bugId=").append(getBugId())
        	.append("]")
            .toString();
    }
}
