package org.openxma.dsl.reference.dto;

import java.io.Serializable;
import org.openxma.dsl.platform.dao.EntityFactory;
import org.openxma.dsl.platform.service.MapperContextHolder;
import java.util.ArrayList;
import java.util.List;
import org.openxma.dsl.platform.service.EntityMapperSupport;
import org.openxma.dsl.reference.dto.CategoryView;
import org.openxma.dsl.reference.dto.ProductView;
import org.openxma.dsl.reference.model.Category;
import org.openxma.dsl.reference.model.Product;

public abstract class CategoryViewGen implements Serializable,EntityMapperSupport {
	private static final long serialVersionUID = -830422988;
	protected List<CategoryView> subCategories;
	protected List<ProductView> products;
	protected Long oid;
	protected String name;
	protected String displayName;
	
	public List<CategoryView> getSubCategories() {
        return this.subCategories;
    }
	
	public void setSubCategories(List<CategoryView> subCategories) {
        this.subCategories=subCategories;
    }
    
    public void addSubCategories(CategoryView subCategories) {
		if (subCategories == null) {
			throw new IllegalArgumentException("parameter 'subCategories' must not be null");
		}
		if (getSubCategories() == null) {
			this.subCategories = new ArrayList<CategoryView>();
		}
		this.subCategories.add(subCategories);
	}
	
	public List<ProductView> getProducts() {
        return this.products;
    }
	
	public void setProducts(List<ProductView> products) {
        this.products=products;
    }
    
    public void addProducts(ProductView products) {
		if (products == null) {
			throw new IllegalArgumentException("parameter 'products' must not be null");
		}
		if (getProducts() == null) {
			this.products = new ArrayList<ProductView>();
		}
		this.products.add(products);
	}
	
	public Long getOid() {
        return this.oid;
    }
	
	public void setOid(Long oid) {
        this.oid=oid;
    }
	
	public String getName() {
        return this.name;
    }
	
	public void setName(String name) {
        this.name=name;
    }
	
	public String getDisplayName() {
        return this.displayName;
    }
	
	public void setDisplayName(String displayName) {
        this.displayName=displayName;
    }
	
    public void mapFrom(Object sourceObject) {
    	if (sourceObject instanceof Category) {
    		Category category = (Category) sourceObject;
    		internalMapFromCategory(category);	
    	} 
    	else if (sourceObject != null) {
			throw new IllegalArgumentException("mapping from object '"+sourceObject+"' is not supported");
		}
	}
	
	public <T> T mapTo(Class<T> target) {
		if (CategoryView.class.equals(target)) { 
			CategoryView categoryView = new CategoryView();
			internalMapToCategoryView(categoryView);		
			return target.cast(categoryView);
		}
		else if (Category.class.equals(target)) {
    		return target.cast(internalMapToCategory(target));
    	}
    	else { 
    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
    	}
    }
	
    public void mapTo(Object targetObject) {
    	if (targetObject instanceof CategoryView) { 
			internalMapToCategoryView(((CategoryView)targetObject));		
		}
    	else if (targetObject instanceof Category) {
    		internalMapToCategory(((Category)targetObject));	
    	} 
    	else if (targetObject != null) { 
			throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
		}
	}
    
    protected void internalMapToCategoryView(CategoryView categoryView) {
		categoryView.setSubCategories(getSubCategories());
		categoryView.setProducts(getProducts());
		categoryView.setOid(getOid());
		categoryView.setName(getName());
		categoryView.setDisplayName(getDisplayName());
    }
    
    protected Category internalMapToCategory(Class<?> target) {
    	EntityFactory<Category> entityFactory = (EntityFactory<Category>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(Category.class);
    	Category category = (Category) 
    		entityFactory.createEntity(null);
    	internalMapToCategory(category);	
    	return category;
    }
    
    protected void internalMapToCategory(Category category) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, category);
			if (getOid() != null) {
				category.setOid(getOid());
			}
			category.setName(getName());
			category.setDisplayName(getDisplayName());
	        if (!isContext("saveCategory") && !isContext("updateCategory")) {
		        if (getSubCategories() != null) {
			        for (CategoryView categoryView : getSubCategories()) {
			    		Category subCategories = categoryView.mapTo(Category.class);
			    		category.addSubCategories(subCategories);
			    	}
		    	}
		        if (getProducts() != null) {
			        for (ProductView productView : getProducts()) {
			    		Product products = productView.mapTo(Product.class);
			    		category.addProducts(products);
			    	}
		    	}
	        } 
	    }
	}
	
	protected boolean isContext(String contextName) {
		return contextName.equals(MapperContextHolder.getContext().get("context"));
	}
	
	protected void internalMapFromCategory(Category category) {
		if (!MapperContextHolder.getContext().containsKey(category)) {
	    	MapperContextHolder.getContext().put(category, this);			
			setOid(category.getOid());
			setName(category.getName());
			setDisplayName(category.getDisplayName());
			if (category.hasSubCategories()) {
				for (Category subCategories : category.getSubCategories()) {
		    		CategoryView categoryView = (CategoryView) MapperContextHolder.getContext().get(subCategories);
		    		if (null==categoryView) {
			        	categoryView = new CategoryView();
						categoryView.mapFrom(subCategories);
			        } 
		        	addSubCategories(categoryView);
		    	}
			}
			if (category.hasProducts()) {
				for (Product products : category.getProducts()) {
		    		ProductView productView = (ProductView) MapperContextHolder.getContext().get(products);
		    		if (null==productView) {
			        	productView = new ProductView();
						productView.mapFrom(products);
			        } 
		        	addProducts(productView);
		    	}
			}
		}
	}
    
    @Override
    public String toString() {
    	return new StringBuilder("CategoryView [")
        	.append("oid=").append(getOid()).append(",")
        	.append("name=").append(getName()).append(",")
        	.append("displayName=").append(getDisplayName())
        	.append("]")
            .toString();
    }
}
