package org.openxma.dsl.reference.dto;

import java.io.Serializable;
import org.openxma.dsl.platform.dao.EntityFactory;
import org.openxma.dsl.platform.service.MapperContextHolder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import org.openxma.dsl.platform.service.EntityMapperSupport;
import org.openxma.dsl.reference.dto.OrderStateView;
import org.openxma.dsl.reference.model.Customer;
import org.openxma.dsl.reference.model.Order;

public abstract class CustomerOrderViewGen implements Serializable,EntityMapperSupport {
	private static final long serialVersionUID = 452911488;
	protected String oid;
	protected String firstName;
	protected String lastName;
	protected Date birthDate;
	protected String emailAddress;
	protected List<OrderStateView> orders;
	
	public String getOid() {
        return this.oid;
    }
	
	public void setOid(String oid) {
        this.oid=oid;
    }
	
	public String getFirstName() {
        return this.firstName;
    }
	
	public void setFirstName(String firstName) {
        this.firstName=firstName;
    }
	
	public String getLastName() {
        return this.lastName;
    }
	
	public void setLastName(String lastName) {
        this.lastName=lastName;
    }
	
	public Date getBirthDate() {
        return this.birthDate;
    }
	
	public void setBirthDate(Date birthDate) {
        this.birthDate=birthDate;
    }
	
	public String getEmailAddress() {
        return this.emailAddress;
    }
	
	public void setEmailAddress(String emailAddress) {
        this.emailAddress=emailAddress;
    }
	
	public List<OrderStateView> getOrders() {
        return this.orders;
    }
	
	public void setOrders(List<OrderStateView> orders) {
        this.orders=orders;
    }
    
    public void addOrders(OrderStateView orders) {
		if (orders == null) {
			throw new IllegalArgumentException("parameter 'orders' must not be null");
		}
		if (getOrders() == null) {
			this.orders = new ArrayList<OrderStateView>();
		}
		this.orders.add(orders);
	}
	
    public void mapFrom(Object sourceObject) {
    	if (sourceObject instanceof Customer) {
    		Customer customer = (Customer) sourceObject;
    		internalMapFromCustomer(customer);	
    	} 
    	else if (sourceObject != null) {
			throw new IllegalArgumentException("mapping from object '"+sourceObject+"' is not supported");
		}
	}
	
	public <T> T mapTo(Class<T> target) {
		if (CustomerOrderView.class.equals(target)) { 
			CustomerOrderView customerOrderView = new CustomerOrderView();
			internalMapToCustomerOrderView(customerOrderView);		
			return target.cast(customerOrderView);
		}
		else if (Customer.class.equals(target)) {
    		return target.cast(internalMapToCustomer(target));
    	}
    	else { 
    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
    	}
    }
	
    public void mapTo(Object targetObject) {
    	if (targetObject instanceof CustomerOrderView) { 
			internalMapToCustomerOrderView(((CustomerOrderView)targetObject));		
		}
    	else if (targetObject instanceof Customer) {
    		internalMapToCustomer(((Customer)targetObject));	
    	} 
    	else if (targetObject != null) { 
			throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
		}
	}
    
    protected void internalMapToCustomerOrderView(CustomerOrderView customerOrderView) {
		customerOrderView.setOid(getOid());
		customerOrderView.setFirstName(getFirstName());
		customerOrderView.setLastName(getLastName());
		customerOrderView.setBirthDate(getBirthDate());
		customerOrderView.setEmailAddress(getEmailAddress());
		customerOrderView.setOrders(getOrders());
    }
    
    protected Customer internalMapToCustomer(Class<?> target) {
    	EntityFactory<Customer> entityFactory = (EntityFactory<Customer>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(Customer.class);
    	Customer customer = (Customer) 
    		entityFactory.createEntity(null);
    	internalMapToCustomer(customer);	
    	return customer;
    }
    
    protected void internalMapToCustomer(Customer customer) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, customer);
			if (getOid() != null) {
				customer.setOid(getOid());
			}
			customer.setFirstName(getFirstName());
			customer.setLastName(getLastName());
			customer.setBirthDate(getBirthDate());
			customer.setEmailAddress(getEmailAddress());
	        if (!isContext("saveCustomer") && !isContext("updateCustomer")) {
		        if (getOrders() != null) {
			        for (OrderStateView orderStateView : getOrders()) {
			    		Order orders = orderStateView.mapTo(Order.class);
			    		customer.addOrders(orders);
			    	}
		    	}
	        } 
	    }
	}
	
	protected boolean isContext(String contextName) {
		return contextName.equals(MapperContextHolder.getContext().get("context"));
	}
	
	protected void internalMapFromCustomer(Customer customer) {
		if (!MapperContextHolder.getContext().containsKey(customer)) {
	    	MapperContextHolder.getContext().put(customer, this);			
			setOid(customer.getOid());
			setFirstName(customer.getFirstName());
			setLastName(customer.getLastName());
			setBirthDate(customer.getBirthDate());
			setEmailAddress(customer.getEmailAddress());
			if (customer.hasOrders()) {
				for (Order orders : customer.getOrders()) {
		    		OrderStateView orderStateView = (OrderStateView) MapperContextHolder.getContext().get(orders);
		    		if (null==orderStateView) {
			        	orderStateView = new OrderStateView();
						orderStateView.mapFrom(orders);
			        } 
		        	addOrders(orderStateView);
		    	}
			}
		}
	}
    
    @Override
    public String toString() {
    	return new StringBuilder("CustomerOrderView [")
        	.append("oid=").append(getOid()).append(",")
        	.append("firstName=").append(getFirstName()).append(",")
        	.append("lastName=").append(getLastName()).append(",")
        	.append("birthDate=").append(getBirthDate()).append(",")
        	.append("emailAddress=").append(getEmailAddress())
        	.append("]")
            .toString();
    }
}
