package org.openxma.dsl.reference.dto;

import java.io.Serializable;
import org.openxma.dsl.platform.dao.EntityFactory;
import org.openxma.dsl.platform.service.MapperContextHolder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import org.openxma.dsl.platform.service.EntityMapperSupport;
import org.openxma.dsl.reference.dto.AddressView;
import org.openxma.dsl.reference.dto.OrderView;
import org.openxma.dsl.reference.model.Address;
import org.openxma.dsl.reference.model.Customer;
import org.openxma.dsl.reference.model.Order;
import org.openxma.dsl.reference.types.Gender;
import org.openxma.dsl.reference.types.valueobject.TelefonNumber;

public abstract class CustomerViewGen implements Serializable,EntityMapperSupport {
	private static final long serialVersionUID = -686128378;
	protected String city;
	protected String country;
	protected String streetName;
	protected List<OrderView> orders;
	protected AddressView invoiceAddress;
	protected AddressView deliveryAddress;
	protected String oid;
	protected Date version;
	protected String firstName;
	protected String lastName;
	protected Gender gender;
	protected String emailAddress;
	protected Date birthDate;
	protected TelefonNumber telefonNumber;
	protected Date lastOrderDate;
	protected String invoiceAddressOid;
	protected String deliveryAddressOid;
	
	public String getCity() {
        return this.city;
    }
	
	public void setCity(String city) {
        this.city=city;
    }
	
	public String getCountry() {
        return this.country;
    }
	
	public void setCountry(String country) {
        this.country=country;
    }
	
	public String getStreetName() {
        return this.streetName;
    }
	
	public void setStreetName(String streetName) {
        this.streetName=streetName;
    }
	
	public List<OrderView> getOrders() {
        return this.orders;
    }
	
	public void setOrders(List<OrderView> orders) {
        this.orders=orders;
    }
    
    public void addOrders(OrderView orders) {
		if (orders == null) {
			throw new IllegalArgumentException("parameter 'orders' must not be null");
		}
		if (getOrders() == null) {
			this.orders = new ArrayList<OrderView>();
		}
		this.orders.add(orders);
	}
	
	public AddressView getInvoiceAddress() {
        return this.invoiceAddress;
    }
	
	public void setInvoiceAddress(AddressView invoiceAddress) {
        this.invoiceAddress=invoiceAddress;
    }
	
	public AddressView getDeliveryAddress() {
        return this.deliveryAddress;
    }
	
	public void setDeliveryAddress(AddressView deliveryAddress) {
        this.deliveryAddress=deliveryAddress;
    }
	
	public String getOid() {
        return this.oid;
    }
	
	public void setOid(String oid) {
        this.oid=oid;
    }
	
	public Date getVersion() {
        return this.version;
    }
	
	public void setVersion(Date version) {
        this.version=version;
    }
	
	public String getFirstName() {
        return this.firstName;
    }
	
	public void setFirstName(String firstName) {
        this.firstName=firstName;
    }
	
	public String getLastName() {
        return this.lastName;
    }
	
	public void setLastName(String lastName) {
        this.lastName=lastName;
    }
	
	public Gender getGender() {
        return this.gender;
    }
	
	public void setGender(Gender gender) {
        this.gender=gender;
    }
	
	public String getEmailAddress() {
        return this.emailAddress;
    }
	
	public void setEmailAddress(String emailAddress) {
        this.emailAddress=emailAddress;
    }
	
	public Date getBirthDate() {
        return this.birthDate;
    }
	
	public void setBirthDate(Date birthDate) {
        this.birthDate=birthDate;
    }
	
	public TelefonNumber getTelefonNumber() {
        return this.telefonNumber;
    }
	
	public void setTelefonNumber(TelefonNumber telefonNumber) {
        this.telefonNumber=telefonNumber;
    }
	
	public Date getLastOrderDate() {
        return this.lastOrderDate;
    }
	
	public void setLastOrderDate(Date lastOrderDate) {
        this.lastOrderDate=lastOrderDate;
    }
	
	public String getInvoiceAddressOid() {
        return this.invoiceAddressOid;
    }
	
	public void setInvoiceAddressOid(String invoiceAddressOid) {
        this.invoiceAddressOid=invoiceAddressOid;
    }
	
	public String getDeliveryAddressOid() {
        return this.deliveryAddressOid;
    }
	
	public void setDeliveryAddressOid(String deliveryAddressOid) {
        this.deliveryAddressOid=deliveryAddressOid;
    }
	
    public void mapFrom(Object sourceObject) {
    	if (sourceObject instanceof Customer) {
    		Customer customer = (Customer) sourceObject;
    		internalMapFromCustomer(customer);	
    		if (customer.getInvoiceAddress()!=null) {
    			setInvoiceAddressOid(customer.getInvoiceAddress().getOid());
    		}
    		if (customer.getDeliveryAddress()!=null) {
    			setDeliveryAddressOid(customer.getDeliveryAddress().getOid());
    		}
    	} 
    	else if (sourceObject instanceof Address) {
    		Address address = (Address) sourceObject;
    		internalMapFromAddress(address);	
    	} 
    	else if (sourceObject != null) {
			throw new IllegalArgumentException("mapping from object '"+sourceObject+"' is not supported");
		}
	}
	
	public <T> T mapTo(Class<T> target) {
		if (CustomerView.class.equals(target)) { 
			CustomerView customerView = new CustomerView();
			internalMapToCustomerView(customerView);		
			return target.cast(customerView);
		}
		else if (Customer.class.equals(target)) {
    		return target.cast(internalMapToCustomer(target));
    	}
		else if (Address.class.equals(target)) {
    		return target.cast(internalMapToAddress(target));
    	}
    	else { 
    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
    	}
    }
	
    public void mapTo(Object targetObject) {
    	if (targetObject instanceof CustomerView) { 
			internalMapToCustomerView(((CustomerView)targetObject));		
		}
    	else if (targetObject instanceof Customer) {
    		internalMapToCustomer(((Customer)targetObject));	
    	} 
    	else if (targetObject instanceof Address) {
    		internalMapToAddress(((Address)targetObject));	
    	} 
    	else if (targetObject != null) { 
			throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
		}
	}
    
    protected void internalMapToCustomerView(CustomerView customerView) {
		customerView.setCity(getCity());
		customerView.setCountry(getCountry());
		customerView.setStreetName(getStreetName());
		customerView.setOrders(getOrders());
		customerView.setInvoiceAddress(getInvoiceAddress());
		customerView.setDeliveryAddress(getDeliveryAddress());
		customerView.setOid(getOid());
		customerView.setVersion(getVersion());
		customerView.setFirstName(getFirstName());
		customerView.setLastName(getLastName());
		customerView.setGender(getGender());
		customerView.setEmailAddress(getEmailAddress());
		customerView.setBirthDate(getBirthDate());
		customerView.setTelefonNumber(getTelefonNumber());
		customerView.setInvoiceAddressOid(getInvoiceAddressOid());
		customerView.setDeliveryAddressOid(getDeliveryAddressOid());
    }
    
    protected Customer internalMapToCustomer(Class<?> target) {
    	EntityFactory<Customer> entityFactory = (EntityFactory<Customer>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(Customer.class);
    	Customer customer = (Customer) 
    		entityFactory.createEntity(null);
    	internalMapToCustomer(customer);	
    	return customer;
    }
    
    protected void internalMapToCustomer(Customer customer) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, customer);
			if (getOid() != null) {
				customer.setOid(getOid());
			}
			customer.setVersion(getVersion());
			customer.setFirstName(getFirstName());
			customer.setLastName(getLastName());
			customer.setGender(getGender());
			customer.setEmailAddress(getEmailAddress());
			customer.setBirthDate(getBirthDate());
			customer.setTelefonNumber(getTelefonNumber());
	        if (!isContext("saveCustomer") && !isContext("updateCustomer")) {
		        if (getOrders() != null) {
			        for (OrderView orderView : getOrders()) {
			    		Order orders = orderView.mapTo(Order.class);
			    		customer.addOrders(orders);
			    	}
		    	}
		        if (getInvoiceAddress() != null) {
		        	Address invoiceAddress = getInvoiceAddress().mapTo(Address.class);
					customer.setInvoiceAddress(invoiceAddress);
		        }
		        if (getDeliveryAddress() != null) {
		        	Address deliveryAddress = getDeliveryAddress().mapTo(Address.class);
					customer.setDeliveryAddress(deliveryAddress);
		        }
	        } 
	    }
	}
	
    protected Address internalMapToAddress(Class<?> target) {
    	EntityFactory<Address> entityFactory = (EntityFactory<Address>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(Address.class);
    	Address address = (Address) 
    		entityFactory.createEntity(null);
    	internalMapToAddress(address);	
    	return address;
    }
    
    protected void internalMapToAddress(Address address) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, address);
			address.setCity(getCity());
			address.setCountry(getCountry());
			address.setStreetName(getStreetName());
			if (getInvoiceAddressOid() != null  && isContext("invoiceAddress")) {
				address.setOid(getInvoiceAddressOid());
			}
			if (getDeliveryAddressOid() != null  && isContext("deliveryAddress")) {
				address.setOid(getDeliveryAddressOid());
			}
	    }
	}
	
	protected boolean isContext(String contextName) {
		return contextName.equals(MapperContextHolder.getContext().get("context"));
	}
	
	protected void internalMapFromCustomer(Customer customer) {
		if (!MapperContextHolder.getContext().containsKey(customer)) {
	    	MapperContextHolder.getContext().put(customer, this);			
			setOid(customer.getOid());
			setVersion(customer.getVersion());
			setFirstName(customer.getFirstName());
			setLastName(customer.getLastName());
			setGender(customer.getGender());
			setEmailAddress(customer.getEmailAddress());
			setBirthDate(customer.getBirthDate());
			setTelefonNumber(customer.getTelefonNumber());
			if (customer.hasOrders()) {
				for (Order orders : customer.getOrders()) {
		    		OrderView orderView = (OrderView) MapperContextHolder.getContext().get(orders);
		    		if (null==orderView) {
			        	orderView = new OrderView();
						orderView.mapFrom(orders);
			        } 
		        	addOrders(orderView);
		    	}
			}
	        if (customer.getInvoiceAddress() != null) {
		        AddressView invoiceAddress = (AddressView) MapperContextHolder.getContext().get(customer.getInvoiceAddress());
		        if (null==invoiceAddress) {
		        	invoiceAddress = new AddressView();
					invoiceAddress.mapFrom(customer.getInvoiceAddress());
		        } 
	        	setInvoiceAddress(invoiceAddress);
	        }
	        if (customer.getDeliveryAddress() != null) {
		        AddressView deliveryAddress = (AddressView) MapperContextHolder.getContext().get(customer.getDeliveryAddress());
		        if (null==deliveryAddress) {
		        	deliveryAddress = new AddressView();
					deliveryAddress.mapFrom(customer.getDeliveryAddress());
		        } 
	        	setDeliveryAddress(deliveryAddress);
	        }
		}
	}
	
	protected void internalMapFromAddress(Address address) {
		if (!MapperContextHolder.getContext().containsKey(address)) {
	    	MapperContextHolder.getContext().put(address, this);			
			setCity(address.getCity());
			setCountry(address.getCountry());
			setStreetName(address.getStreetName());
			setInvoiceAddressOid(address.getOid());
			setDeliveryAddressOid(address.getOid());
		}
	}
    
    @Override
    public String toString() {
    	return new StringBuilder("CustomerView [")
        	.append("city=").append(getCity()).append(",")
        	.append("country=").append(getCountry()).append(",")
        	.append("streetName=").append(getStreetName()).append(",")
        	.append("oid=").append(getOid()).append(",")
        	.append("version=").append(getVersion()).append(",")
        	.append("firstName=").append(getFirstName()).append(",")
        	.append("lastName=").append(getLastName()).append(",")
        	.append("gender=").append(getGender()).append(",")
        	.append("emailAddress=").append(getEmailAddress()).append(",")
        	.append("birthDate=").append(getBirthDate()).append(",")
        	.append("telefonNumber=").append(getTelefonNumber()).append(",")
        	.append("lastOrderDate=").append(getLastOrderDate()).append(",")
        	.append("invoiceAddressOid=").append(getInvoiceAddressOid()).append(",")
        	.append("deliveryAddressOid=").append(getDeliveryAddressOid())
        	.append("]")
            .toString();
    }
}
