package org.openxma.dsl.reference.dto;

import java.io.Serializable;
import org.openxma.dsl.platform.dao.EntityFactory;
import org.openxma.dsl.platform.service.MapperContextHolder;
import java.util.Date;
import org.openxma.dsl.platform.service.EntityMapperSupport;
import org.openxma.dsl.reference.model.Order;
import org.openxma.dsl.reference.model.OrderItem;
import org.openxma.dsl.reference.model.Product;
import org.openxma.dsl.reference.types.valueobject.ProductId;

public abstract class OrderItemViewGen implements Serializable,EntityMapperSupport {
	private static final long serialVersionUID = 1070503018;
	protected String oid;
	protected Date version;
	protected Integer itemNo;
	protected Long quantity;
	protected String orderOid;
	protected ProductId productOid;
	
	public String getOid() {
        return this.oid;
    }
	
	public void setOid(String oid) {
        this.oid=oid;
    }
	
	public Date getVersion() {
        return this.version;
    }
	
	public void setVersion(Date version) {
        this.version=version;
    }
	
	public Integer getItemNo() {
        return this.itemNo;
    }
	
	public void setItemNo(Integer itemNo) {
        this.itemNo=itemNo;
    }
	
	public Long getQuantity() {
        return this.quantity;
    }
	
	public void setQuantity(Long quantity) {
        this.quantity=quantity;
    }
	
	public String getOrderOid() {
        return this.orderOid;
    }
	
	public void setOrderOid(String orderOid) {
        this.orderOid=orderOid;
    }
	
	public ProductId getProductOid() {
        return this.productOid;
    }
	
	public void setProductOid(ProductId productOid) {
        this.productOid=productOid;
    }
	
    public void mapFrom(Object sourceObject) {
    	if (sourceObject instanceof OrderItem) {
    		OrderItem orderItem = (OrderItem) sourceObject;
    		internalMapFromOrderItem(orderItem);	
    		if (orderItem.getOrder()!=null) {
    			setOrderOid(orderItem.getOrder().getOid());
    		}
    		if (orderItem.getProduct()!=null) {
    			setProductOid(orderItem.getProduct().getOid());
    		}
    	} 
    	else if (sourceObject instanceof Order) {
    		Order order = (Order) sourceObject;
    		internalMapFromOrder(order);	
    	} 
    	else if (sourceObject instanceof Product) {
    		Product product = (Product) sourceObject;
    		internalMapFromProduct(product);	
    	} 
    	else if (sourceObject != null) {
			throw new IllegalArgumentException("mapping from object '"+sourceObject+"' is not supported");
		}
	}
	
	public <T> T mapTo(Class<T> target) {
		if (OrderItemView.class.equals(target)) { 
			OrderItemView orderItemView = new OrderItemView();
			internalMapToOrderItemView(orderItemView);		
			return target.cast(orderItemView);
		}
		else if (OrderItem.class.equals(target)) {
    		return target.cast(internalMapToOrderItem(target));
    	}
		else if (Order.class.equals(target)) {
    		return target.cast(internalMapToOrder(target));
    	}
		else if (Product.class.equals(target)) {
    		return target.cast(internalMapToProduct(target));
    	}
    	else { 
    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
    	}
    }
	
    public void mapTo(Object targetObject) {
    	if (targetObject instanceof OrderItemView) { 
			internalMapToOrderItemView(((OrderItemView)targetObject));		
		}
    	else if (targetObject instanceof OrderItem) {
    		internalMapToOrderItem(((OrderItem)targetObject));	
    	} 
    	else if (targetObject instanceof Order) {
    		internalMapToOrder(((Order)targetObject));	
    	} 
    	else if (targetObject instanceof Product) {
    		internalMapToProduct(((Product)targetObject));	
    	} 
    	else if (targetObject != null) { 
			throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
		}
	}
    
    protected void internalMapToOrderItemView(OrderItemView orderItemView) {
		orderItemView.setOid(getOid());
		orderItemView.setVersion(getVersion());
		orderItemView.setItemNo(getItemNo());
		orderItemView.setQuantity(getQuantity());
		orderItemView.setOrderOid(getOrderOid());
		orderItemView.setProductOid(getProductOid());
    }
    
    protected OrderItem internalMapToOrderItem(Class<?> target) {
    	EntityFactory<OrderItem> entityFactory = (EntityFactory<OrderItem>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(OrderItem.class);
    	OrderItem orderItem = (OrderItem) 
    		entityFactory.createEntity(null);
    	internalMapToOrderItem(orderItem);	
    	return orderItem;
    }
    
    protected void internalMapToOrderItem(OrderItem orderItem) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, orderItem);
			if (getOid() != null) {
				orderItem.setOid(getOid());
			}
			orderItem.setVersion(getVersion());
			orderItem.setItemNo(getItemNo());
			orderItem.setQuantity(getQuantity());
	    }
	}
	
    protected Order internalMapToOrder(Class<?> target) {
    	EntityFactory<Order> entityFactory = (EntityFactory<Order>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(Order.class);
    	Order order = (Order) 
    		entityFactory.createEntity(null);
    	internalMapToOrder(order);	
    	return order;
    }
    
    protected void internalMapToOrder(Order order) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, order);
			if (getOrderOid() != null  && isContext("order")) {
				order.setOid(getOrderOid());
			}
	    }
	}
	
    protected Product internalMapToProduct(Class<?> target) {
    	EntityFactory<Product> entityFactory = (EntityFactory<Product>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(Product.class);
    	Product product = (Product) 
    		entityFactory.createEntity(null);
    	internalMapToProduct(product);	
    	return product;
    }
    
    protected void internalMapToProduct(Product product) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, product);
			if (getProductOid() != null  && isContext("product")) {
				product.setOid(getProductOid());
			}
	    }
	}
	
	protected boolean isContext(String contextName) {
		return contextName.equals(MapperContextHolder.getContext().get("context"));
	}
	
	protected void internalMapFromOrderItem(OrderItem orderItem) {
		if (!MapperContextHolder.getContext().containsKey(orderItem)) {
	    	MapperContextHolder.getContext().put(orderItem, this);			
			setOid(orderItem.getOid());
			setVersion(orderItem.getVersion());
			setItemNo(orderItem.getItemNo());
			setQuantity(orderItem.getQuantity());
		}
	}
	
	protected void internalMapFromOrder(Order order) {
		if (!MapperContextHolder.getContext().containsKey(order)) {
	    	MapperContextHolder.getContext().put(order, this);			
			setOrderOid(order.getOid());
		}
	}
	
	protected void internalMapFromProduct(Product product) {
		if (!MapperContextHolder.getContext().containsKey(product)) {
	    	MapperContextHolder.getContext().put(product, this);			
			setProductOid(product.getOid());
		}
	}
    
    @Override
    public String toString() {
    	return new StringBuilder("OrderItemView [")
        	.append("oid=").append(getOid()).append(",")
        	.append("version=").append(getVersion()).append(",")
        	.append("itemNo=").append(getItemNo()).append(",")
        	.append("quantity=").append(getQuantity()).append(",")
        	.append("orderOid=").append(getOrderOid()).append(",")
        	.append("productOid=").append(getProductOid())
        	.append("]")
            .toString();
    }
}
