package org.openxma.dsl.reference.dto;

import java.io.Serializable;
import org.openxma.dsl.platform.dao.EntityFactory;
import org.openxma.dsl.platform.service.MapperContextHolder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import org.openxma.dsl.platform.service.EntityMapperSupport;
import org.openxma.dsl.reference.dto.CustomerOrderView;
import org.openxma.dsl.reference.dto.OrderItemQuantityView;
import org.openxma.dsl.reference.model.Customer;
import org.openxma.dsl.reference.model.Order;
import org.openxma.dsl.reference.model.OrderItem;
import org.openxma.dsl.reference.types.valueobject.OrderNumber;

public abstract class OrderStateViewGen implements Serializable,EntityMapperSupport {
	private static final long serialVersionUID = 1570511469;
	protected Long maxOrderItemCount;
	protected String oid;
	protected OrderNumber orderNumber;
	protected Date deliveryDate;
	protected String storeId;
	protected Date placementDate;
	protected String orderState;
	protected CustomerOrderView customer;
	protected List<OrderItemQuantityView> orderItems;
	
	public Long getMaxOrderItemCount() {
        return this.maxOrderItemCount;
    }
	
	public void setMaxOrderItemCount(Long maxOrderItemCount) {
        this.maxOrderItemCount=maxOrderItemCount;
    }
	
	public String getOid() {
        return this.oid;
    }
	
	public void setOid(String oid) {
        this.oid=oid;
    }
	
	public OrderNumber getOrderNumber() {
        return this.orderNumber;
    }
	
	public void setOrderNumber(OrderNumber orderNumber) {
        this.orderNumber=orderNumber;
    }
	
	public Date getDeliveryDate() {
        return this.deliveryDate;
    }
	
	public void setDeliveryDate(Date deliveryDate) {
        this.deliveryDate=deliveryDate;
    }
	
	public String getStoreId() {
        return this.storeId;
    }
	
	public void setStoreId(String storeId) {
        this.storeId=storeId;
    }
	
	public Date getPlacementDate() {
        return this.placementDate;
    }
	
	public void setPlacementDate(Date placementDate) {
        this.placementDate=placementDate;
    }
	
	public String getOrderState() {
        return this.orderState;
    }
	
	public void setOrderState(String orderState) {
        this.orderState=orderState;
    }
	
	public CustomerOrderView getCustomer() {
        return this.customer;
    }
	
	public void setCustomer(CustomerOrderView customer) {
        this.customer=customer;
    }
	
	public List<OrderItemQuantityView> getOrderItems() {
        return this.orderItems;
    }
	
	public void setOrderItems(List<OrderItemQuantityView> orderItems) {
        this.orderItems=orderItems;
    }
    
    public void addOrderItems(OrderItemQuantityView orderItems) {
		if (orderItems == null) {
			throw new IllegalArgumentException("parameter 'orderItems' must not be null");
		}
		if (getOrderItems() == null) {
			this.orderItems = new ArrayList<OrderItemQuantityView>();
		}
		this.orderItems.add(orderItems);
	}
	
    public void mapFrom(Object sourceObject) {
    	if (sourceObject instanceof Order) {
    		Order order = (Order) sourceObject;
    		internalMapFromOrder(order);	
    	} 
    	else if (sourceObject != null) {
			throw new IllegalArgumentException("mapping from object '"+sourceObject+"' is not supported");
		}
	}
	
	public <T> T mapTo(Class<T> target) {
		if (OrderStateView.class.equals(target)) { 
			OrderStateView orderStateView = new OrderStateView();
			internalMapToOrderStateView(orderStateView);		
			return target.cast(orderStateView);
		}
		else if (Order.class.equals(target)) {
    		return target.cast(internalMapToOrder(target));
    	}
    	else { 
    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
    	}
    }
	
    public void mapTo(Object targetObject) {
    	if (targetObject instanceof OrderStateView) { 
			internalMapToOrderStateView(((OrderStateView)targetObject));		
		}
    	else if (targetObject instanceof Order) {
    		internalMapToOrder(((Order)targetObject));	
    	} 
    	else if (targetObject != null) { 
			throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
		}
	}
    
    protected void internalMapToOrderStateView(OrderStateView orderStateView) {
		orderStateView.setOid(getOid());
		orderStateView.setOrderNumber(getOrderNumber());
		orderStateView.setDeliveryDate(getDeliveryDate());
		orderStateView.setStoreId(getStoreId());
		orderStateView.setPlacementDate(getPlacementDate());
		orderStateView.setOrderState(getOrderState());
		orderStateView.setCustomer(getCustomer());
		orderStateView.setOrderItems(getOrderItems());
    }
    
    protected Order internalMapToOrder(Class<?> target) {
    	EntityFactory<Order> entityFactory = (EntityFactory<Order>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(Order.class);
    	Order order = (Order) 
    		entityFactory.createEntity(null);
    	internalMapToOrder(order);	
    	return order;
    }
    
    protected void internalMapToOrder(Order order) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, order);
			if (getOid() != null) {
				order.setOid(getOid());
			}
			order.setOrderNumber(getOrderNumber());
			order.setDeliveryDate(getDeliveryDate());
			order.setStoreId(getStoreId());
			order.setPlacementDate(getPlacementDate());
			order.setOrderState(getOrderState());
	        if (!isContext("saveOrder") && !isContext("updateOrder")) {
		        if (getCustomer() != null) {
		        	Customer customer = getCustomer().mapTo(Customer.class);
					order.setCustomer(customer);
		        }
		        if (getOrderItems() != null) {
			        for (OrderItemQuantityView orderItemQuantityView : getOrderItems()) {
			    		OrderItem orderItems = orderItemQuantityView.mapTo(OrderItem.class);
			    		order.addOrderItems(orderItems);
			    	}
		    	}
	        } 
	    }
	}
	
	protected boolean isContext(String contextName) {
		return contextName.equals(MapperContextHolder.getContext().get("context"));
	}
	
	protected void internalMapFromOrder(Order order) {
		if (!MapperContextHolder.getContext().containsKey(order)) {
	    	MapperContextHolder.getContext().put(order, this);			
			setOid(order.getOid());
			setOrderNumber(order.getOrderNumber());
			setDeliveryDate(order.getDeliveryDate());
			setStoreId(order.getStoreId());
			setPlacementDate(order.getPlacementDate());
			setOrderState(order.getOrderState());
	        if (order.getCustomer() != null) {
		        CustomerOrderView customer = (CustomerOrderView) MapperContextHolder.getContext().get(order.getCustomer());
		        if (null==customer) {
		        	customer = new CustomerOrderView();
					customer.mapFrom(order.getCustomer());
		        } 
	        	setCustomer(customer);
	        }
			if (order.hasOrderItems()) {
				for (OrderItem orderItems : order.getOrderItems()) {
		    		OrderItemQuantityView orderItemQuantityView = (OrderItemQuantityView) MapperContextHolder.getContext().get(orderItems);
		    		if (null==orderItemQuantityView) {
			        	orderItemQuantityView = new OrderItemQuantityView();
						orderItemQuantityView.mapFrom(orderItems);
			        } 
		        	addOrderItems(orderItemQuantityView);
		    	}
			}
		}
	}
    
    @Override
    public String toString() {
    	return new StringBuilder("OrderStateView [")
        	.append("maxOrderItemCount=").append(getMaxOrderItemCount()).append(",")
        	.append("oid=").append(getOid()).append(",")
        	.append("orderNumber=").append(getOrderNumber()).append(",")
        	.append("deliveryDate=").append(getDeliveryDate()).append(",")
        	.append("storeId=").append(getStoreId()).append(",")
        	.append("placementDate=").append(getPlacementDate()).append(",")
        	.append("orderState=").append(getOrderState())
        	.append("]")
            .toString();
    }
}
