package org.openxma.dsl.reference.dto;

import java.io.Serializable;
import org.openxma.dsl.platform.dao.EntityFactory;
import org.openxma.dsl.platform.service.MapperContextHolder;
import org.openxma.dsl.platform.service.EntityMapperSupport;
import org.openxma.dsl.reference.model.Product;
import org.openxma.dsl.reference.types.valueobject.ProductId;

public abstract class ProductIdViewGen implements Serializable,EntityMapperSupport {
	private static final long serialVersionUID = -1870120908;
	protected String productDisplayName;
	protected ProductId oid;
	
	public String getProductDisplayName() {
        return this.productDisplayName;
    }
	
	public void setProductDisplayName(String productDisplayName) {
        this.productDisplayName=productDisplayName;
    }
	
	public ProductId getOid() {
        return this.oid;
    }
	
	public void setOid(ProductId oid) {
        this.oid=oid;
    }
	
    public void mapFrom(Object sourceObject) {
    	if (sourceObject instanceof Product) {
    		Product product = (Product) sourceObject;
    		internalMapFromProduct(product);	
    	} 
    	else if (sourceObject != null) {
			throw new IllegalArgumentException("mapping from object '"+sourceObject+"' is not supported");
		}
	}
	
	public <T> T mapTo(Class<T> target) {
		if (ProductIdView.class.equals(target)) { 
			ProductIdView productIdView = new ProductIdView();
			internalMapToProductIdView(productIdView);		
			return target.cast(productIdView);
		}
		else if (Product.class.equals(target)) {
    		return target.cast(internalMapToProduct(target));
    	}
    	else { 
    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
    	}
    }
	
    public void mapTo(Object targetObject) {
    	if (targetObject instanceof ProductIdView) { 
			internalMapToProductIdView(((ProductIdView)targetObject));		
		}
    	else if (targetObject instanceof Product) {
    		internalMapToProduct(((Product)targetObject));	
    	} 
    	else if (targetObject != null) { 
			throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
		}
	}
    
    protected void internalMapToProductIdView(ProductIdView productIdView) {
		productIdView.setOid(getOid());
    }
    
    protected Product internalMapToProduct(Class<?> target) {
    	EntityFactory<Product> entityFactory = (EntityFactory<Product>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(Product.class);
    	Product product = (Product) 
    		entityFactory.createEntity(null);
    	internalMapToProduct(product);	
    	return product;
    }
    
    protected void internalMapToProduct(Product product) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, product);
			if (getOid() != null) {
				product.setOid(getOid());
			}
	    }
	}
	
	protected void internalMapFromProduct(Product product) {
		if (!MapperContextHolder.getContext().containsKey(product)) {
	    	MapperContextHolder.getContext().put(product, this);			
			setOid(product.getOid());
		}
	}
    
    @Override
    public String toString() {
    	return new StringBuilder("ProductIdView [")
        	.append("productDisplayName=").append(getProductDisplayName()).append(",")
        	.append("oid=").append(getOid())
        	.append("]")
            .toString();
    }
}
