package org.openxma.dsl.reference.dto;

import java.io.Serializable;
import org.openxma.dsl.platform.dao.EntityFactory;
import org.openxma.dsl.platform.service.MapperContextHolder;
import java.util.Date;
import org.openxma.dsl.platform.service.EntityMapperSupport;
import org.openxma.dsl.reference.model.Product;
import org.openxma.dsl.reference.model.ProductImage;
import org.openxma.dsl.reference.types.valueobject.ProductId;

public abstract class ProductImageViewGen implements Serializable,EntityMapperSupport {
	private static final long serialVersionUID = -1466006550;
	protected String oid;
	protected Date version;
	protected String name;
	protected String fileName;
	protected Integer sizeX;
	protected Integer sizeY;
	protected ProductId productOid;
	
	public String getOid() {
        return this.oid;
    }
	
	public void setOid(String oid) {
        this.oid=oid;
    }
	
	public Date getVersion() {
        return this.version;
    }
	
	public void setVersion(Date version) {
        this.version=version;
    }
	
	public String getName() {
        return this.name;
    }
	
	public void setName(String name) {
        this.name=name;
    }
	
	public String getFileName() {
        return this.fileName;
    }
	
	public void setFileName(String fileName) {
        this.fileName=fileName;
    }
	
	public Integer getSizeX() {
        return this.sizeX;
    }
	
	public void setSizeX(Integer sizeX) {
        this.sizeX=sizeX;
    }
	
	public Integer getSizeY() {
        return this.sizeY;
    }
	
	public void setSizeY(Integer sizeY) {
        this.sizeY=sizeY;
    }
	
	public ProductId getProductOid() {
        return this.productOid;
    }
	
	public void setProductOid(ProductId productOid) {
        this.productOid=productOid;
    }
	
    public void mapFrom(Object sourceObject) {
    	if (sourceObject instanceof ProductImage) {
    		ProductImage productImage = (ProductImage) sourceObject;
    		internalMapFromProductImage(productImage);	
    		if (productImage.getProduct()!=null) {
    			setProductOid(productImage.getProduct().getOid());
    		}
    	} 
    	else if (sourceObject instanceof Product) {
    		Product product = (Product) sourceObject;
    		internalMapFromProduct(product);	
    	} 
    	else if (sourceObject != null) {
			throw new IllegalArgumentException("mapping from object '"+sourceObject+"' is not supported");
		}
	}
	
	public <T> T mapTo(Class<T> target) {
		if (ProductImageView.class.equals(target)) { 
			ProductImageView productImageView = new ProductImageView();
			internalMapToProductImageView(productImageView);		
			return target.cast(productImageView);
		}
		else if (ProductImage.class.equals(target)) {
    		return target.cast(internalMapToProductImage(target));
    	}
		else if (Product.class.equals(target)) {
    		return target.cast(internalMapToProduct(target));
    	}
    	else { 
    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
    	}
    }
	
    public void mapTo(Object targetObject) {
    	if (targetObject instanceof ProductImageView) { 
			internalMapToProductImageView(((ProductImageView)targetObject));		
		}
    	else if (targetObject instanceof ProductImage) {
    		internalMapToProductImage(((ProductImage)targetObject));	
    	} 
    	else if (targetObject instanceof Product) {
    		internalMapToProduct(((Product)targetObject));	
    	} 
    	else if (targetObject != null) { 
			throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
		}
	}
    
    protected void internalMapToProductImageView(ProductImageView productImageView) {
		productImageView.setOid(getOid());
		productImageView.setVersion(getVersion());
		productImageView.setName(getName());
		productImageView.setFileName(getFileName());
		productImageView.setSizeX(getSizeX());
		productImageView.setSizeY(getSizeY());
		productImageView.setProductOid(getProductOid());
    }
    
    protected ProductImage internalMapToProductImage(Class<?> target) {
    	EntityFactory<ProductImage> entityFactory = (EntityFactory<ProductImage>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(ProductImage.class);
    	ProductImage productImage = (ProductImage) 
    		entityFactory.createEntity(null);
    	internalMapToProductImage(productImage);	
    	return productImage;
    }
    
    protected void internalMapToProductImage(ProductImage productImage) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, productImage);
			if (getOid() != null) {
				productImage.setOid(getOid());
			}
			productImage.setVersion(getVersion());
			productImage.setName(getName());
			productImage.setFileName(getFileName());
			productImage.setSizeX(getSizeX());
			productImage.setSizeY(getSizeY());
	    }
	}
	
    protected Product internalMapToProduct(Class<?> target) {
    	EntityFactory<Product> entityFactory = (EntityFactory<Product>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(Product.class);
    	Product product = (Product) 
    		entityFactory.createEntity(null);
    	internalMapToProduct(product);	
    	return product;
    }
    
    protected void internalMapToProduct(Product product) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, product);
			if (getProductOid() != null  && isContext("product")) {
				product.setOid(getProductOid());
			}
	    }
	}
	
	protected boolean isContext(String contextName) {
		return contextName.equals(MapperContextHolder.getContext().get("context"));
	}
	
	protected void internalMapFromProductImage(ProductImage productImage) {
		if (!MapperContextHolder.getContext().containsKey(productImage)) {
	    	MapperContextHolder.getContext().put(productImage, this);			
			setOid(productImage.getOid());
			setVersion(productImage.getVersion());
			setName(productImage.getName());
			setFileName(productImage.getFileName());
			setSizeX(productImage.getSizeX());
			setSizeY(productImage.getSizeY());
		}
	}
	
	protected void internalMapFromProduct(Product product) {
		if (!MapperContextHolder.getContext().containsKey(product)) {
	    	MapperContextHolder.getContext().put(product, this);			
			setProductOid(product.getOid());
		}
	}
    
    @Override
    public String toString() {
    	return new StringBuilder("ProductImageView [")
        	.append("oid=").append(getOid()).append(",")
        	.append("version=").append(getVersion()).append(",")
        	.append("name=").append(getName()).append(",")
        	.append("fileName=").append(getFileName()).append(",")
        	.append("sizeX=").append(getSizeX()).append(",")
        	.append("sizeY=").append(getSizeY()).append(",")
        	.append("productOid=").append(getProductOid())
        	.append("]")
            .toString();
    }
}
