package org.openxma.dsl.reference.dto;

import java.io.Serializable;
import org.openxma.dsl.platform.dao.EntityFactory;
import org.openxma.dsl.platform.service.MapperContextHolder;
import java.util.Date;
import org.openxma.dsl.platform.service.EntityMapperSupport;
import org.openxma.dsl.reference.model.Product;
import org.openxma.dsl.reference.model.Supplier;
import org.openxma.dsl.reference.types.valueobject.ProductId;

public abstract class ProductViewGen implements Serializable,EntityMapperSupport {
	private static final long serialVersionUID = 366491962;
	protected ProductId oid;
	protected Date version;
	protected String name;
	protected Integer unitPrice;
	protected Integer unitOnStock;
	protected Integer unitOnOrder;
	protected String supplierOid;
	
	public ProductId getOid() {
        return this.oid;
    }
	
	public void setOid(ProductId oid) {
        this.oid=oid;
    }
	
	public Date getVersion() {
        return this.version;
    }
	
	public void setVersion(Date version) {
        this.version=version;
    }
	
	public String getName() {
        return this.name;
    }
	
	public void setName(String name) {
        this.name=name;
    }
	
	public Integer getUnitPrice() {
        return this.unitPrice;
    }
	
	public void setUnitPrice(Integer unitPrice) {
        this.unitPrice=unitPrice;
    }
	
	public Integer getUnitOnStock() {
        return this.unitOnStock;
    }
	
	public void setUnitOnStock(Integer unitOnStock) {
        this.unitOnStock=unitOnStock;
    }
	
	public Integer getUnitOnOrder() {
        return this.unitOnOrder;
    }
	
	public void setUnitOnOrder(Integer unitOnOrder) {
        this.unitOnOrder=unitOnOrder;
    }
	
	public String getSupplierOid() {
        return this.supplierOid;
    }
	
	public void setSupplierOid(String supplierOid) {
        this.supplierOid=supplierOid;
    }
	
    public void mapFrom(Object sourceObject) {
    	if (sourceObject instanceof Product) {
    		Product product = (Product) sourceObject;
    		internalMapFromProduct(product);	
    		if (product.getSupplier()!=null) {
    			setSupplierOid(product.getSupplier().getOid());
    		}
    	} 
    	else if (sourceObject instanceof Supplier) {
    		Supplier supplier = (Supplier) sourceObject;
    		internalMapFromSupplier(supplier);	
    	} 
    	else if (sourceObject != null) {
			throw new IllegalArgumentException("mapping from object '"+sourceObject+"' is not supported");
		}
	}
	
	public <T> T mapTo(Class<T> target) {
		if (ProductView.class.equals(target)) { 
			ProductView productView = new ProductView();
			internalMapToProductView(productView);		
			return target.cast(productView);
		}
		else if (Product.class.equals(target)) {
    		return target.cast(internalMapToProduct(target));
    	}
		else if (Supplier.class.equals(target)) {
    		return target.cast(internalMapToSupplier(target));
    	}
    	else { 
    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
    	}
    }
	
    public void mapTo(Object targetObject) {
    	if (targetObject instanceof ProductView) { 
			internalMapToProductView(((ProductView)targetObject));		
		}
    	else if (targetObject instanceof Product) {
    		internalMapToProduct(((Product)targetObject));	
    	} 
    	else if (targetObject instanceof Supplier) {
    		internalMapToSupplier(((Supplier)targetObject));	
    	} 
    	else if (targetObject != null) { 
			throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
		}
	}
    
    protected void internalMapToProductView(ProductView productView) {
		productView.setOid(getOid());
		productView.setVersion(getVersion());
		productView.setName(getName());
		productView.setUnitPrice(getUnitPrice());
		productView.setUnitOnStock(getUnitOnStock());
		productView.setUnitOnOrder(getUnitOnOrder());
		productView.setSupplierOid(getSupplierOid());
    }
    
    protected Product internalMapToProduct(Class<?> target) {
    	EntityFactory<Product> entityFactory = (EntityFactory<Product>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(Product.class);
    	Product product = (Product) 
    		entityFactory.createEntity(null);
    	internalMapToProduct(product);	
    	return product;
    }
    
    protected void internalMapToProduct(Product product) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, product);
			if (getOid() != null) {
				product.setOid(getOid());
			}
			product.setVersion(getVersion());
			product.setName(getName());
			product.setUnitPrice(getUnitPrice());
			product.setUnitOnStock(getUnitOnStock());
			product.setUnitOnOrder(getUnitOnOrder());
	    }
	}
	
    protected Supplier internalMapToSupplier(Class<?> target) {
    	EntityFactory<Supplier> entityFactory = (EntityFactory<Supplier>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(Supplier.class);
    	Supplier supplier = (Supplier) 
    		entityFactory.createEntity(null);
    	internalMapToSupplier(supplier);	
    	return supplier;
    }
    
    protected void internalMapToSupplier(Supplier supplier) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, supplier);
			if (getSupplierOid() != null  && isContext("supplier")) {
				supplier.setOid(getSupplierOid());
			}
	    }
	}
	
	protected boolean isContext(String contextName) {
		return contextName.equals(MapperContextHolder.getContext().get("context"));
	}
	
	protected void internalMapFromProduct(Product product) {
		if (!MapperContextHolder.getContext().containsKey(product)) {
	    	MapperContextHolder.getContext().put(product, this);			
			setOid(product.getOid());
			setVersion(product.getVersion());
			setName(product.getName());
			setUnitPrice(product.getUnitPrice());
			setUnitOnStock(product.getUnitOnStock());
			setUnitOnOrder(product.getUnitOnOrder());
		}
	}
	
	protected void internalMapFromSupplier(Supplier supplier) {
		if (!MapperContextHolder.getContext().containsKey(supplier)) {
	    	MapperContextHolder.getContext().put(supplier, this);			
			setSupplierOid(supplier.getOid());
		}
	}
    
    @Override
    public String toString() {
    	return new StringBuilder("ProductView [")
        	.append("oid=").append(getOid()).append(",")
        	.append("version=").append(getVersion()).append(",")
        	.append("name=").append(getName()).append(",")
        	.append("unitPrice=").append(getUnitPrice()).append(",")
        	.append("unitOnStock=").append(getUnitOnStock()).append(",")
        	.append("unitOnOrder=").append(getUnitOnOrder()).append(",")
        	.append("supplierOid=").append(getSupplierOid())
        	.append("]")
            .toString();
    }
}
