package org.openxma.dsl.reference.dto;

import java.io.Serializable;
import org.openxma.dsl.platform.dao.EntityFactory;
import org.openxma.dsl.platform.service.MapperContextHolder;
import java.util.Date;
import org.openxma.dsl.platform.service.EntityMapperSupport;
import org.openxma.dsl.reference.model.Customer;
import org.openxma.dsl.reference.model.Order;
import org.openxma.dsl.reference.types.valueobject.OrderNumber;

public abstract class SingleOrderViewGen implements Serializable,EntityMapperSupport {
	private static final long serialVersionUID = -1904141917;
	protected String customerOid;
	protected String firstName;
	protected String lastName;
	protected String orderOid;
	protected OrderNumber orderNumber;
	protected Date deliveryDate;
	protected String storeId;
	
	public String getCustomerOid() {
        return this.customerOid;
    }
	
	public void setCustomerOid(String customerOid) {
        this.customerOid=customerOid;
    }
	
	public String getFirstName() {
        return this.firstName;
    }
	
	public void setFirstName(String firstName) {
        this.firstName=firstName;
    }
	
	public String getLastName() {
        return this.lastName;
    }
	
	public void setLastName(String lastName) {
        this.lastName=lastName;
    }
	
	public String getOrderOid() {
        return this.orderOid;
    }
	
	public void setOrderOid(String orderOid) {
        this.orderOid=orderOid;
    }
	
	public OrderNumber getOrderNumber() {
        return this.orderNumber;
    }
	
	public void setOrderNumber(OrderNumber orderNumber) {
        this.orderNumber=orderNumber;
    }
	
	public Date getDeliveryDate() {
        return this.deliveryDate;
    }
	
	public void setDeliveryDate(Date deliveryDate) {
        this.deliveryDate=deliveryDate;
    }
	
	public String getStoreId() {
        return this.storeId;
    }
	
	public void setStoreId(String storeId) {
        this.storeId=storeId;
    }
	
    public void mapFrom(Object sourceObject) {
    	if (sourceObject instanceof Customer) {
    		Customer customer = (Customer) sourceObject;
    		internalMapFromCustomer(customer);	
    	} 
    	else if (sourceObject instanceof Order) {
    		Order order = (Order) sourceObject;
    		internalMapFromOrder(order);	
    	} 
    	else if (sourceObject != null) {
			throw new IllegalArgumentException("mapping from object '"+sourceObject+"' is not supported");
		}
	}
	
	public <T> T mapTo(Class<T> target) {
		if (SingleOrderView.class.equals(target)) { 
			SingleOrderView singleOrderView = new SingleOrderView();
			internalMapToSingleOrderView(singleOrderView);		
			return target.cast(singleOrderView);
		}
		else if (Customer.class.equals(target)) {
    		return target.cast(internalMapToCustomer(target));
    	}
		else if (Order.class.equals(target)) {
    		return target.cast(internalMapToOrder(target));
    	}
    	else { 
    		throw new IllegalArgumentException("mapping to class '"+target+"' is not supported");
    	}
    }
	
    public void mapTo(Object targetObject) {
    	if (targetObject instanceof SingleOrderView) { 
			internalMapToSingleOrderView(((SingleOrderView)targetObject));		
		}
    	else if (targetObject instanceof Customer) {
    		internalMapToCustomer(((Customer)targetObject));	
    	} 
    	else if (targetObject instanceof Order) {
    		internalMapToOrder(((Order)targetObject));	
    	} 
    	else if (targetObject != null) { 
			throw new IllegalArgumentException("mapping to object '"+targetObject+"' is not supported");
		}
	}
    
    protected void internalMapToSingleOrderView(SingleOrderView singleOrderView) {
		singleOrderView.setCustomerOid(getCustomerOid());
		singleOrderView.setFirstName(getFirstName());
		singleOrderView.setLastName(getLastName());
		singleOrderView.setOrderOid(getOrderOid());
		singleOrderView.setOrderNumber(getOrderNumber());
		singleOrderView.setDeliveryDate(getDeliveryDate());
		singleOrderView.setStoreId(getStoreId());
    }
    
    protected Customer internalMapToCustomer(Class<?> target) {
    	EntityFactory<Customer> entityFactory = (EntityFactory<Customer>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(Customer.class);
    	Customer customer = (Customer) 
    		entityFactory.createEntity(null);
    	internalMapToCustomer(customer);	
    	return customer;
    }
    
    protected void internalMapToCustomer(Customer customer) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, customer);
			if (getCustomerOid() != null) {
				customer.setOid(getCustomerOid());
			}
			customer.setFirstName(getFirstName());
			customer.setLastName(getLastName());
	    }
	}
	
    protected Order internalMapToOrder(Class<?> target) {
    	EntityFactory<Order> entityFactory = (EntityFactory<Order>) 
    		MapperContextHolder.getEntityFactoryRegistry().getEntityFactory(Order.class);
    	Order order = (Order) 
    		entityFactory.createEntity(null);
    	internalMapToOrder(order);	
    	return order;
    }
    
    protected void internalMapToOrder(Order order) {
    	if (!MapperContextHolder.getContext().containsKey(this)) {
	    	MapperContextHolder.getContext().put(this, order);
			if (getOrderOid() != null) {
				order.setOid(getOrderOid());
			}
			order.setOrderNumber(getOrderNumber());
			order.setDeliveryDate(getDeliveryDate());
			order.setStoreId(getStoreId());
	    }
	}
	
	protected void internalMapFromCustomer(Customer customer) {
		if (!MapperContextHolder.getContext().containsKey(customer)) {
	    	MapperContextHolder.getContext().put(customer, this);			
			setCustomerOid(customer.getOid());
			setFirstName(customer.getFirstName());
			setLastName(customer.getLastName());
		}
	}
	
	protected void internalMapFromOrder(Order order) {
		if (!MapperContextHolder.getContext().containsKey(order)) {
	    	MapperContextHolder.getContext().put(order, this);			
			setOrderOid(order.getOid());
			setOrderNumber(order.getOrderNumber());
			setDeliveryDate(order.getDeliveryDate());
			setStoreId(order.getStoreId());
		}
	}
    
    @Override
    public String toString() {
    	return new StringBuilder("SingleOrderView [")
        	.append("customerOid=").append(getCustomerOid()).append(",")
        	.append("firstName=").append(getFirstName()).append(",")
        	.append("lastName=").append(getLastName()).append(",")
        	.append("orderOid=").append(getOrderOid()).append(",")
        	.append("orderNumber=").append(getOrderNumber()).append(",")
        	.append("deliveryDate=").append(getDeliveryDate()).append(",")
        	.append("storeId=").append(getStoreId())
        	.append("]")
            .toString();
    }
}
