package org.openxma.dsl.reference.model.impl;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import org.openxma.dsl.reference.model.Bug;

public abstract class BugGenImpl implements Bug {
	protected Long oid;
	protected String bugId;
	protected Set<Bug> relatedBugs;
	protected Set<Bug> relatedByBugs;
    
	public Long getOid() {
		return this.oid;
	}
	
	public void setOid(Long oid) {
		this.oid = oid;
	}

	public String getBugId() {
		return this.bugId;
	}
	
	public void setBugId(String bugId) {
		this.bugId = bugId;
	}

	public Set<Bug> getRelatedBugs() {
		if (this.relatedBugs == null) {
            this.relatedBugs = new HashSet<Bug>();
        }
        return Collections.unmodifiableSet(this.relatedBugs);
	}
	
	public boolean hasRelatedBugs() {
		return relatedBugs != null && !relatedBugs.isEmpty();
	}

	public void addRelatedBugs(Bug relatedBugs) {
		if (relatedBugs == null) {
			throw new IllegalArgumentException("parameter 'relatedBugs' must not be null");
		}
		if (!getRelatedBugs().contains(relatedBugs)) {
			this.relatedBugs.add(relatedBugs);
			relatedBugs.addRelatedByBugs(this);
		}
	}

	public void removeRelatedBugs(Bug relatedBugs) {
		if (relatedBugs == null) {
			throw new IllegalArgumentException("parameter 'relatedBugs' must not be null");
		}
		if (getRelatedBugs().contains(relatedBugs)) {
            this.relatedBugs.remove(relatedBugs);
			relatedBugs.removeRelatedByBugs(this);
		}
	}

    public void removeAllRelatedBugs() {
        if (this.relatedBugs != null) {
        	for (Bug bug : new HashSet<Bug>(this.relatedBugs)) {
				removeRelatedBugs(bug);
			}
        }
    }

	public Set<Bug> getRelatedByBugs() {
		if (this.relatedByBugs == null) {
            this.relatedByBugs = new HashSet<Bug>();
        }
        return Collections.unmodifiableSet(this.relatedByBugs);
	}
	
	public boolean hasRelatedByBugs() {
		return relatedByBugs != null && !relatedByBugs.isEmpty();
	}

	public void addRelatedByBugs(Bug relatedByBugs) {
		if (relatedByBugs == null) {
			throw new IllegalArgumentException("parameter 'relatedByBugs' must not be null");
		}
		if (!getRelatedByBugs().contains(relatedByBugs)) {
			this.relatedByBugs.add(relatedByBugs);
			relatedByBugs.addRelatedBugs(this);
		}
	}

	public void removeRelatedByBugs(Bug relatedByBugs) {
		if (relatedByBugs == null) {
			throw new IllegalArgumentException("parameter 'relatedByBugs' must not be null");
		}
		if (getRelatedByBugs().contains(relatedByBugs)) {
            this.relatedByBugs.remove(relatedByBugs);
			relatedByBugs.removeRelatedBugs(this);
		}
	}

    public void removeAllRelatedByBugs() {
        if (this.relatedByBugs != null) {
            this.relatedByBugs.clear();
        }
    }
	
    @Override
    public int hashCode() {
    	final int prime = 31;
		int result = 1;
		result = prime * result + ((getOid() == null) ? 0 : getOid().hashCode());
		return result;
    }

    @Override
    public boolean equals(Object obj) {
    	if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof Bug)) {
			return false;
		}
		Bug other = (Bug) obj;
		if (getOid() == null) {
			if (other.getOid() != null) {
				return false;
			}
		} else if (!getOid().equals(other.getOid())) {
			return false;
		}
		return true;
    }
	
    @Override
    public String toString() {
    	return new StringBuilder("Bug [")
        	.append("oid=").append(getOid()).append(",")
        	.append("bugId=").append(getBugId())
        	.append("]")
            .toString();
    }
}
