package org.openxma.dsl.reference.model.impl;

import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;
import org.openxma.dsl.reference.base.model.impl.BaseEntityImpl;
import org.openxma.dsl.reference.model.Customer;
import org.openxma.dsl.reference.model.Order;
import org.openxma.dsl.reference.model.OrderItem;
import org.openxma.dsl.reference.types.valueobject.OrderNumber;

public abstract class OrderGenImpl extends BaseEntityImpl implements Order {
	protected OrderNumber orderNumber;
	protected String storeId;
	protected Date placementDate;
	protected Date deliveryDate;
	protected String orderState;
	protected Integer taxes;
	protected Double totalAmount;
	protected Customer customer;
	protected Set<OrderItem> orderItems;
    
	public OrderNumber getOrderNumber() {
		return this.orderNumber;
	}
	
	public void setOrderNumber(OrderNumber orderNumber) {
		this.orderNumber = orderNumber;
	}

	public String getStoreId() {
		return this.storeId;
	}
	
	public void setStoreId(String storeId) {
		this.storeId = storeId;
	}

	public Date getPlacementDate() {
		return this.placementDate;
	}
	
	public void setPlacementDate(Date placementDate) {
		this.placementDate = placementDate;
	}

	public Date getDeliveryDate() {
		return this.deliveryDate;
	}
	
	public void setDeliveryDate(Date deliveryDate) {
		this.deliveryDate = deliveryDate;
	}

	public String getOrderState() {
		return this.orderState;
	}
	
	public void setOrderState(String orderState) {
		this.orderState = orderState;
	}

	public Integer getTaxes() {
		return this.taxes;
	}
	
	public void setTaxes(Integer taxes) {
		this.taxes = taxes;
	}

	public Double getTotalAmount() {
		return this.totalAmount;
	}
	
	public void setTotalAmount(Double totalAmount) {
		this.totalAmount = totalAmount;
	}

	public Customer getCustomer() {
		return this.customer;
	}
	
	public void setCustomer(Customer customer) {
		if (this.customer != customer) {
			if (this.customer != null) {
				this.customer.removeOrders(this);
			}
			this.customer = customer;
			if (customer != null) {
				customer.addOrders(this);
			}
		}
	}

	public Set<OrderItem> getOrderItems() {
		if (this.orderItems == null) {
            this.orderItems = new HashSet<OrderItem>();
        }
        return Collections.unmodifiableSet(this.orderItems);
	}
	
	public boolean hasOrderItems() {
		return orderItems != null && !orderItems.isEmpty();
	}

	public void addOrderItems(OrderItem orderItems) {
		if (orderItems == null) {
			throw new IllegalArgumentException("parameter 'orderItems' must not be null");
		}
		if (!getOrderItems().contains(orderItems)) {
			this.orderItems.add(orderItems);
			orderItems.setOrder(this);
		}
	}

	public void removeOrderItems(OrderItem orderItems) {
		if (orderItems == null) {
			throw new IllegalArgumentException("parameter 'orderItems' must not be null");
		}
		if (getOrderItems().contains(orderItems)) {
            this.orderItems.remove(orderItems);
			orderItems.setOrder(null);
		}
	}

    public void removeAllOrderItems() {
        if (this.orderItems != null) {
            this.orderItems.clear();
        }
    }
	
    @Override
    public String toString() {
    	return new StringBuilder("Order [")
    		.append(super.toString()).append(",")
        	.append("orderNumber=").append(getOrderNumber()).append(",")
        	.append("storeId=").append(getStoreId()).append(",")
        	.append("placementDate=").append(getPlacementDate()).append(",")
        	.append("deliveryDate=").append(getDeliveryDate()).append(",")
        	.append("orderState=").append(getOrderState()).append(",")
        	.append("taxes=").append(getTaxes()).append(",")
        	.append("totalAmount=").append(getTotalAmount())
        	.append("]")
            .toString();
    }
}
