package org.openxma.dsl.reference.model.impl;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import org.openxma.dsl.reference.base.model.impl.BaseEntityImpl;
import org.openxma.dsl.reference.model.Role;
import org.openxma.dsl.reference.model.RoleGroup;

public abstract class RoleGenImpl extends BaseEntityImpl implements Role {
	protected String name;
	protected Set<RoleGroup> parentsGroups;
    
	public String getName() {
		return this.name;
	}
	
	public void setName(String name) {
		this.name = name;
	}

	public Set<RoleGroup> getParentsGroups() {
		if (this.parentsGroups == null) {
            this.parentsGroups = new HashSet<RoleGroup>();
        }
        return Collections.unmodifiableSet(this.parentsGroups);
	}
	
	public boolean hasParentsGroups() {
		return parentsGroups != null && !parentsGroups.isEmpty();
	}

	public void addParentsGroups(RoleGroup parentsGroups) {
		if (parentsGroups == null) {
			throw new IllegalArgumentException("parameter 'parentsGroups' must not be null");
		}
		if (!getParentsGroups().contains(parentsGroups)) {
			this.parentsGroups.add(parentsGroups);
			parentsGroups.addRoles(this);
		}
	}

	public void removeParentsGroups(RoleGroup parentsGroups) {
		if (parentsGroups == null) {
			throw new IllegalArgumentException("parameter 'parentsGroups' must not be null");
		}
		if (getParentsGroups().contains(parentsGroups)) {
            this.parentsGroups.remove(parentsGroups);
			parentsGroups.removeRoles(this);
		}
	}

    public void removeAllParentsGroups() {
        if (this.parentsGroups != null) {
            this.parentsGroups.clear();
        }
    }
	
    @Override
    public String toString() {
    	return new StringBuilder("Role [")
    		.append(super.toString()).append(",")
        	.append("name=").append(getName())
        	.append("]")
            .toString();
    }
}
