package org.openxma.dsl.reference.model.impl;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import org.openxma.dsl.reference.model.Role;
import org.openxma.dsl.reference.model.RoleGroup;
import org.openxma.dsl.reference.model.impl.RoleImpl;

public abstract class RoleGroupGenImpl extends RoleImpl implements RoleGroup {
	protected Set<Role> roles;
    
	public Set<Role> getRoles() {
		if (this.roles == null) {
            this.roles = new HashSet<Role>();
        }
        return Collections.unmodifiableSet(this.roles);
	}
	
	public boolean hasRoles() {
		return roles != null && !roles.isEmpty();
	}

	public void addRoles(Role roles) {
		if (roles == null) {
			throw new IllegalArgumentException("parameter 'roles' must not be null");
		}
		if (!getRoles().contains(roles)) {
			this.roles.add(roles);
			roles.addParentsGroups(this);
		}
	}

	public void removeRoles(Role roles) {
		if (roles == null) {
			throw new IllegalArgumentException("parameter 'roles' must not be null");
		}
		if (getRoles().contains(roles)) {
            this.roles.remove(roles);
			roles.removeParentsGroups(this);
		}
	}

    public void removeAllRoles() {
        if (this.roles != null) {
        	for (Role role : new HashSet<Role>(this.roles)) {
				removeRoles(role);
			}
        }
    }
	
    @Override
    public String toString() {
    	return new StringBuilder("RoleGroup [")
    		.append(super.toString()).append(",")
        	.append("]")
            .toString();
    }
}
