package org.openxma.dsl.reference.service.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.openxma.dsl.platform.service.Mapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.openxma.dsl.reference.dao.SupplierDao;
import org.openxma.dsl.reference.dto.SupplierView;
import org.openxma.dsl.reference.model.Supplier;
import org.openxma.dsl.reference.service.SupplierService;
import org.springframework.dao.OptimisticLockingFailureException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

public abstract class SupplierServiceGenImpl implements SupplierService {
	protected final Log logger = LogFactory.getLog(getClass());
	@Autowired(required=false)
	protected Mapper mapper;
	protected SupplierDao supplierDao;
	
	@Autowired
	public void setSupplierDao(SupplierDao supplierDao) {
	    this.supplierDao = supplierDao;
	}
	
	@Transactional
	public SupplierView save(SupplierView supplierView) {
		Assert.notNull(supplierView, "parameter 'supplierView' must not be null");
        Supplier supplier = this.mapper.createAndMapOne(supplierView, Supplier.class,"saveSupplier");
        this.supplierDao.saveOrUpdate(supplier);
        return this.mapper.mapOne(supplier,new SupplierView());
	}
	
	@Transactional(readOnly = true)
	public SupplierView loadSupplierView(String oid) {
		Assert.notNull(oid, "parameter 'oid' must not be null");
        return this.mapper.createAndMapOne(this.supplierDao.load(oid),SupplierView.class);
	}
	
	@Transactional
	public void update(SupplierView supplierView) {
		Assert.notNull(supplierView, "parameter 'supplierView' must not be null");
		Supplier mappedSupplier = this.mapper.createAndMapOne(supplierView, Supplier.class);
        Supplier supplier = this.supplierDao.load(mappedSupplier.getOid());
        if (!supplier.getVersion().equals(mappedSupplier.getVersion())) {
        	throw new OptimisticLockingFailureException("Entity 'Supplier' was updated or deleted by another transaction.");
        }
        this.mapper.mapOne(supplierView, supplier,"updateSupplier");
	}
	
	@Transactional
	public void deleteSupplier(String oid) {
		Assert.notNull(oid, "parameter 'oid' must not be null");
        this.supplierDao.delete(oid);
	}
	
}
