package org.openxma.dsl.reference.validation;

import org.openxma.dsl.platform.validation.Validators;
import org.springframework.validation.Errors;

import org.openxma.dsl.reference.model.Customer;
import org.springframework.validation.ValidationUtils;

public abstract class CustomerGenValidator extends Validators {
	
	public boolean isFirstNameRequired(Customer customer) {
		return Boolean.TRUE;
	}
	
	public boolean isLastNameRequired(Customer customer) {
		return Boolean.TRUE;
	}
	
	public boolean isGenderRequired(Customer customer) {
		return Boolean.TRUE;
	}
	
	public boolean isEmailAddressRequired(Customer customer) {
		return Boolean.TRUE;
	}
	
	public boolean isBirthDateRequired(Customer customer) {
		return Boolean.TRUE;
	}
	
	public boolean isTelefonNumberRequired(Customer customer) {
		return Boolean.FALSE;
	}
	
	public boolean isInvoiceAddressRequired(Customer customer) {
		return Boolean.TRUE;
	}
	
	public boolean isDeliveryAddressRequired(Customer customer) {
		return Boolean.TRUE;
	}
		
	@SuppressWarnings("unchecked")
	public boolean supports(Class clazz) {
		return Customer.class.isAssignableFrom(clazz);
	}
	
	public Errors validate(Customer customer) {
		Errors errors = createErrors(customer);
		validate(customer,errors);
		return errors;
	}

	public void validate(Object object, Errors errors) {
		Customer customer = (Customer) object;
		if (isFirstNameRequired(customer)) {
			ValidationUtils.rejectIfEmptyOrWhitespace(errors, "firstName", "field.required");
		}
		if (isLastNameRequired(customer)) {
			ValidationUtils.rejectIfEmptyOrWhitespace(errors, "lastName", "field.required");
		}
		if (isGenderRequired(customer)) {
			ValidationUtils.rejectIfEmpty(errors, "gender", "field.required");
		}
		if (isEmailAddressRequired(customer)) {
			ValidationUtils.rejectIfEmptyOrWhitespace(errors, "emailAddress", "field.required");
		}
		if (isBirthDateRequired(customer)) {
			ValidationUtils.rejectIfEmpty(errors, "birthDate", "field.required");
		}
		if (isTelefonNumberRequired(customer)) {
			ValidationUtils.rejectIfEmpty(errors, "telefonNumber", "field.required");
		}
		if (isInvoiceAddressRequired(customer)) {
			ValidationUtils.rejectIfEmpty(errors, "invoiceAddress", "field.required");
		}
		if (isDeliveryAddressRequired(customer)) {
			ValidationUtils.rejectIfEmpty(errors, "deliveryAddress", "field.required");
		}
		rejectIfMaxLength(errors,"firstName",customer.getFirstName(),25);
		rejectIfMaxLength(errors,"lastName",customer.getLastName(),25);
		rejectIfMaxLength(errors,"emailAddress",customer.getEmailAddress(),40);
	}
	
}
