package org.openxma.dsl.reference.xma.customeredit.server;

import at.spardat.xma.page.*;
import at.spardat.xma.component.*;
import at.spardat.xma.mdl.*;
import at.spardat.xma.rpc.RemoteCall;
import at.spardat.xma.rpc.RemoteReply;

/**
 * This class implements the generic server part of CustomerEditForm.
 * It implements the models of the page.
 *
 */
public abstract class CustomerEditFormGen extends PageServer {
    Page1 page1;
    Page2 page2;
    Page3 page3;
    WModel[] widgetModels;


    /**
     * Constructs the CustomerEditFormGen.
     * This constructor is called by the runtime. Do not remove!
     *
     * @param component the component this page belongs to
     */
    public CustomerEditFormGen(ComponentServer component) {
        super(component, false);
        createModels();
    }

    /**
     * Create and initialize the WidgetModels of the page.
     */
    public void createModels() {

        widgetModels = new WModel[] {  };
    }

    /**
     * Returns an array of all WidgetModels of this page.
     * @return array of WidgetModels. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public WModel[] getWModels() {
        return widgetModels;
    }

    /**
     * Replaces the model array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWModels(WModel[] models) {
        widgetModels=models;
    }

    /**
     * Returns a unique id to identify the type of this Page.
     * @return the typeId
     */
    public short getTypeId() {
        return CustomerEdit.CustomerEditFormClass;
    }

    /**
     * Returns the component this page belongs to, casted to its actual type.
     * @return The Component. Never null.
     */
    public CustomerEdit getTypedComponent() {
        return (CustomerEdit) getComponent();
    }

    /**
     * Copies values of all attached widget models to attributes of the BusinessData instance variables.
     * Copies them for this page and all contained NotebookPages and statically embedded Pages
     * (embedded via XMAContainer in guidesigner)
     */
    public void modelToInstanceData() {
        if(page1!=null) {
            page1.modelToInstanceData();
        }
        if(page2!=null) {
            page2.modelToInstanceData();
        }
        if(page3!=null) {
            page3.modelToInstanceData();
        }
    }

    /**
     * Copies attribute values of the BusinessData instance variables into their attached widget models.
     * This includes simple widget models and tables.
     * Copies them for this page and all contained NotebookPages and statically embedded Pages
     * (embedded via XMAContainer in guidesigner)
     */
    public void instanceDataToModel() {
        if(page1!=null) {
            page1.instanceDataToModel();
        }
        if(page2!=null) {
            page2.instanceDataToModel();
        }
        if(page3!=null) {
            page3.instanceDataToModel();
        }
    }

    /**
     * Sets the given subpage. The method automatically chooses the
     * correct member variable to set.
     * @param child the subpage to set.
     */
    public void setChild(PageServer child) {
        if(child instanceof Page1) {
            page1 = (Page1) child;
        } else if(child instanceof Page2) {
            page2 = (Page2) child;
        } else if(child instanceof Page3) {
            page3 = (Page3) child;
        }
    }

    /**
     * Removes the given subpage. The method automatically chooses the
     * correct member variable to clear.
     * @param child the subpage to set.
     */
    public void removeChild(PageServer child) {
        if(child instanceof Page1) {
            page1 = null;
        } else if(child instanceof Page2) {
            page2 = null;
        } else if(child instanceof Page3) {
            page3 = null;
        }
    }

    abstract public void readCustomer(RemoteCall call, RemoteReply reply);
    abstract public void saveCustomer(RemoteCall call, RemoteReply reply);
    abstract public void cancel(RemoteCall call, RemoteReply reply);
    abstract public void selectOrder(RemoteCall call, RemoteReply reply);
}
