package org.openxma.dsl.reference.xma.order.server;

import at.spardat.xma.page.*;
import at.spardat.xma.component.*;
import at.spardat.xma.mdl.*;
import at.spardat.enterprise.util.*;
import at.spardat.xma.mdl.table.*;
import at.spardat.xma.mdl.simple.*;
import java.util.*;
import at.spardat.xma.rpc.RemoteCall;
import at.spardat.xma.rpc.RemoteReply;

/**
 * This class implements the generic server part of OrderSearchForm.
 * It implements the models of the page.
 *
 */
public abstract class OrderSearchFormGen extends PageServer {
    ISimpleWM orderState_orderNumber;
    ISimpleWM customer;
    ITableWM orderTable;
    static final int ORDERTABLE_ORDERNUMBER = 0;
    static final int ORDERTABLE_STOREID = 1;
    static final int ORDERTABLE_PLACEMENTDATE = 2;
    static final int ORDERTABLE_DELIVERYDATE = 3;
    static final int ORDERTABLE_ORDERSTATE = 4;
    WModel[] widgetModels;


    /**
     * Constructs the OrderSearchFormGen.
     * This constructor is called by the runtime. Do not remove!
     *
     * @param component the component this page belongs to
     */
    public OrderSearchFormGen(ComponentServer component) {
        super(component, false);
        createModels();
    }

    /**
     * Create and initialize the WidgetModels of the page.
     */
    public void createModels() {
        orderState_orderNumber = new SimpleWM((short) 0, Types.T_STRING, this);
        customer = new SimpleWM((short) 1, Types.T_STRING, this);
        orderTable = new TableWM((short) 2,this,5,TableWM.S_NULL|TableWM.S_ONE_WAY);

        widgetModels = new WModel[] { (WModel) orderState_orderNumber, (WModel) customer, (WModel) orderTable };
    }

    /**
     * Returns an array of all WidgetModels of this page.
     * @return array of WidgetModels. The returned array is not a copy, therefore
     *          it must not be modified.
     */
    public WModel[] getWModels() {
        return widgetModels;
    }

    /**
     * Replaces the model array generated by guidesigner.
     * May only be called by the runtime. Do not call this method!
     */
    protected void setWModels(WModel[] models) {
        widgetModels=models;
    }

    /**
     * Returns a unique id to identify the type of this Page.
     * @return the typeId
     */
    public short getTypeId() {
        return Order.OrderSearchFormClass;
    }

    /**
     * Returns the component this page belongs to, casted to its actual type.
     * @return The Component. Never null.
     */
    public Order getTypedComponent() {
        return (Order) getComponent();
    }

    /**
     * Copies values of all attached widget models to attributes of the BusinessData instance variables.
     * Copies them for this page and all contained NotebookPages and statically embedded Pages
     * (embedded via XMAContainer in guidesigner)
     */
    public void modelToInstanceData() {
        atomicWMToBusinessData(getTypedComponent().getOrderState());
    }

    /**
     * Copies attribute values of the BusinessData instance variables into their attached widget models.
     * This includes simple widget models and tables.
     * Copies them for this page and all contained NotebookPages and statically embedded Pages
     * (embedded via XMAContainer in guidesigner)
     */
    public void instanceDataToModel() {
        businessDataToAtomicWM(getTypedComponent().getOrderState());
        orderTableFill();
    }

    /**
     * Copies values of widget models to attributes of the provided BusinessData objects.
     */
    public void atomicWMToBusinessData(org.openxma.dsl.reference.dto.OrderStateView orderState) {
        orderState.setOrderNumber(org.openxma.dsl.reference.types.valueobject.OrderNumber.newOrNull(orderState_orderNumber.toString()));
    }

    /**
     * Copies attribute values of the provided BusinessDatas into widget models.
     */
    public void businessDataToAtomicWM(org.openxma.dsl.reference.dto.OrderStateView orderState) {
          orderState_orderNumber.set(org.openxma.dsl.platform.valueobject.ValueObject.valueOrNull(orderState.getOrderNumber()));
    }

    /**
     * This is a support method to create the table cells of one row from one or more BusinessDatas.
     * You may overwrite this method if you need a custom mapping from BusinessData attributes
     * to table columns that you were unable to specify in the UI designer.
     *
     * @return the returned Object[] is needed in the constructor of a <tt>TableRow</tt>.
     */
    public Object[] orderTableCreateCells(org.openxma.dsl.reference.dto.OrderView orders) {
        Object []  cells = new Object[5];
        cells[0] = org.openxma.dsl.platform.valueobject.ValueObject.valueOrNull(orders.getOrderNumber());        cells[1] = orders.getStoreId();
        cells[2] = orders.getPlacementDate();
        cells[3] = orders.getDeliveryDate();
        cells[4] = orders.getOrderState();
        return cells;
    }

    /**
     * This method creates a TableRow object and adds it to the end of the table.
     * The key of the row is drawn from the key attribute specified in the UI designer.
     * The image id is resolved by calling orderTableGetImageId. The cells of the table
     * are taken from a call to orderTableCreateCells.
     */
    public TableRow orderTableAddRow (org.openxma.dsl.reference.dto.OrderView orders) {
        return new TableRow ((TableWM)orderTable, orders.getOid().toString(), orderTableCreateCells (orders), orderTableGetImageFor (orders));
    }

    /**
     * This method is called from <tt>orderTableAddRow</tt> to figure out an image
     * id for a particular row. If you do not overwrite this method, the table row
     * does not have an image id. 
     */
    public int orderTableGetImageFor (org.openxma.dsl.reference.dto.OrderView orders) {
        return 0;
    }

    /**
     * This method iterates over the collection and calls <tt>orderTableAddRow</tt> for
     * each element.
     */
    public void orderTableFill () {
        orderTable.clear();
        for (Iterator iter = getTypedComponent().getOrders().iterator(); iter.hasNext();) {
            orderTableAddRow ((org.openxma.dsl.reference.dto.OrderView) iter.next());
        }
    }

    /**
     * Sets the given subpage. The method automatically chooses the
     * correct member variable to set.
     * @param child the subpage to set.
     */
    public void setChild(PageServer child) {
       // no subpages - nothing to do
    }

    /**
     * Removes the given subpage. The method automatically chooses the
     * correct member variable to clear.
     * @param child the subpage to set.
     */
    public void removeChild(PageServer child) {
       // no subpages - nothing to do
    }

    abstract public void findOrder(RemoteCall call, RemoteReply reply);
    abstract public void selectOrder(RemoteCall call, RemoteReply reply);
}
