package org.openxma.dsl.reference.xma.web;

import java.util.Date;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.openxma.dsl.reference.dao.AddressDao;
import org.openxma.dsl.reference.dao.CustomerDao;
import org.openxma.dsl.reference.dao.OrderDao;
import org.openxma.dsl.reference.dao.OrderItemDao;
import org.openxma.dsl.reference.dao.ProductDao;
import org.openxma.dsl.reference.dao.SupplierDao;
import org.openxma.dsl.reference.model.Address;
import org.openxma.dsl.reference.model.Customer;
import org.openxma.dsl.reference.model.Order;
import org.openxma.dsl.reference.model.Product;
import org.openxma.dsl.reference.model.Supplier;
import org.openxma.dsl.reference.types.Gender;
import org.openxma.dsl.reference.types.valueobject.OrderNumber;
import org.openxma.dsl.reference.types.valueobject.TelefonNumber;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionTemplate;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

public class TestDataLoader implements ServletContextListener{

	public void contextInitialized(ServletContextEvent sce) {
		WebApplicationContext webApplicationContext = WebApplicationContextUtils.getWebApplicationContext(sce.getServletContext());
		final CustomerDao customerDao = (CustomerDao) webApplicationContext.getBeansOfType(CustomerDao.class).values().iterator().next();
		final OrderDao orderDao = (OrderDao) webApplicationContext.getBeansOfType(OrderDao.class).values().iterator().next();
		final OrderItemDao orderItemDao = (OrderItemDao) webApplicationContext.getBeansOfType(OrderItemDao.class).values().iterator().next();
		final ProductDao productDao = (ProductDao) webApplicationContext.getBeansOfType(ProductDao.class).values().iterator().next();
		final SupplierDao supplierDao = (SupplierDao) webApplicationContext.getBeansOfType(SupplierDao.class).values().iterator().next();
		final AddressDao addressDao = (AddressDao) webApplicationContext.getBeansOfType(AddressDao.class).values().iterator().next();
		PlatformTransactionManager transactionManager = (PlatformTransactionManager) webApplicationContext.getBean("transactionManager");
		for (int i = 0; i < 100; i++) {
			final Address invoiceAddress =addressDao.create();
			invoiceAddress.setCity("VIENNA");
			invoiceAddress.setCountry("AUSTRIA");
			invoiceAddress.setStreetName("Geiselbergst");
			invoiceAddress.setStreetNumber("25");
			invoiceAddress.setZip("1110");
			
			final Address deliveryAddress = addressDao.create();
			deliveryAddress.setCity("VIENNA");
			
			final Customer customer = newCustomer(invoiceAddress,customerDao, "Last"+i);
			customer.setDeliveryAddress(deliveryAddress);
			Order order1 = orderDao.create(customer,new OrderNumber("AA-XX-1231"+i),"MAH-1"+i,new Date(), "NEW",1d);
			Order order2 = orderDao.create(customer,new OrderNumber("AA-XX-1232"+i),"MAH-1"+i,new Date(), "NEW",1d);
			final Supplier supplier = supplierDao.create("TestCompany"+System.nanoTime());
			final Product product = productDao.create(supplier,"product"+i, 1);
			orderItemDao.create(order1,product,1, new Long(1));
			orderItemDao.create(order2,product,1, new Long(1));
			customer.addOrders(order1);
			customer.addOrders(order2);
			new TransactionTemplate(transactionManager).execute(new TransactionCallback(){
				public Object doInTransaction(TransactionStatus status) {
					addressDao.saveOrUpdate(invoiceAddress);
					addressDao.saveOrUpdate(deliveryAddress);
					supplierDao.saveOrUpdate(supplier);
					productDao.saveOrUpdate(product);
					customerDao.saveOrUpdate(customer);
					return null;
				}});
		}
	}
	
	protected Customer newCustomer(Address invoiceAddress,CustomerDao customerDao,String lastName) {
		Customer customer = customerDao.create(invoiceAddress,invoiceAddress,"First", lastName, Gender.MALE ,"first@last.at", new Date());
		customer.setTelefonNumber(new TelefonNumber(1234));
		return customer;
	}

	public void contextDestroyed(ServletContextEvent sce) {
	}
}
